@@ -19,12 +19,12 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "cgraphicsitem.h"
-#include "cgraphicsmasicoitem.h"
 #include "cgraphicstextitem.h"
 #include "cgraphicsproxywidget.h"
 #include "cdrawscene.h"
 #include "widgets/ctextedit.h"
 #include "frame/cgraphicsview.h"
+#include "cviewmanagement.h"
 #include "drawItems/cgraphicsitemselectedmgr.h"
 #include "application.h"
 #include "global.h"
@@ -41,13 +41,34 @@
 #include "cgraphicscutitem.h"
 #include "cgraphicsellipseitem.h"
 #include "cgraphicslineitem.h"
-#include "cgraphicsmasicoitem.h"
 #include "cgraphicspenitem.h"
 #include "cgraphicspolygonalstaritem.h"
 #include "cgraphicspolygonitem.h"
 #include "cgraphicstextitem.h"
 #include "cgraphicstriangleitem.h"
 #include "cpictureitem.h"
+#include "cattributeitemwidget.h"
+#include "cgraphicslayer.h"
+#include "blurwidget.h"
+
+REGISTITEMCLASS(CGraphicsPenItem, PenType)
+REGISTITEMCLASS(CPictureItem, PictureType)
+class CGraphicsItem::CGraphicsItem_private
+{
+public:
+    explicit CGraphicsItem_private(CGraphicsItem *pItem): _item(pItem) {}
+    static QMap<int, QString> &registedClassNameMap()
+    {
+        static QMap<int, QString> *s_map = nullptr;
+        if (s_map == nullptr) {
+            s_map = new QMap<int, QString>;
+        }
+        return *s_map;
+    }
+
+private:
+    CGraphicsItem *_item;
+};
 
 const int inccW = 10;
 bool CGraphicsItem::paintSelectedBorderLine = true;
@@ -87,35 +108,32 @@ QPainterPath CGraphicsItem::getGraphicsItemShapePathByOrg(const QPainterPath &or
 CGraphicsItem *CGraphicsItem::creatItemInstance(int itemType, const CGraphicsUnit &data)
 {
     CGraphicsItem *item = nullptr;
-    if (RectType == itemType) {
-        item = new CGraphicsRectItem;
-    } else if (EllipseType == itemType) {
-        item = new CGraphicsEllipseItem;
-    } else if (TriangleType == itemType) {
-        item = new CGraphicsTriangleItem;
-    } else if (PolygonType == itemType) {
-        item = new CGraphicsPolygonItem;
-    } else if (PolygonalStarType == itemType) {
-        item = new CGraphicsPolygonalStarItem;
-    } else if (LineType == itemType) {
-        item = new CGraphicsLineItem;
-    } else if (TextType == itemType) {
-        item = new CGraphicsTextItem;
-    } else if (PictureType == itemType) {
-        item = new CPictureItem;
-    } else if (PenType == itemType) {
-        item = new CGraphicsPenItem;
-    }/* else if (BlurType == itemType) {
-        item = new CGraphicsMasicoItem;
-    }*/ else if (CutType == itemType) {
-        item = new CGraphicsCutItem;
-    } else {
-        qDebug() << "!!!!!!!!!!!!!!!!!!!!!!unknoewd type !!!!!!!!!!!! = " << itemType;
+    auto registerItems = CGraphicsItem_private::registedClassNameMap();
+    auto itf = registerItems.find(itemType);
+    if (itf != registerItems.end()) {
+        item =  ClassObjectFactory::creatObject<CGraphicsItem>(itf.value());
     }
 
     if (item != nullptr)
         item->loadGraphicsUnit(data);
 
+    if ((PictureType == itemType || itemType == PenType) && item != nullptr) {
+        QPixmap pix = item->rasterSelf();
+        auto jLay = new JDynamicLayer(pix.toImage());
+        auto gemo = new JGeomeCommand(item->pos(), item->drawRotation(), item->zValue(), item->boundingRectTruly(), item->transform());
+        gemo->setLayer(jLay);
+        jLay->appendComand(gemo, true, false);
+
+        jLay->setBlocked(false);
+        if (itemType == PictureType) {
+            jLay->setBlocked(true);
+            jLay->setLayerType(JDynamicLayer::EImageType);
+        }
+
+        delete item;
+        item = jLay;
+    }
+
     return item;
 }
 
@@ -137,9 +155,9 @@ CGraphicsItem *CGraphicsItem::zItem(const QList<CGraphicsItem *> &pBzItems, int
     }
 
     if (wantZitemTp == -1) {
-        result = CDrawScene::returnSortZItems(allBzItems, CDrawScene::EAesSort).first();
+        result = PageScene::returnSortZItems(allBzItems, PageScene::EAesSort).first();
     } else if (wantZitemTp == -2) {
-        result = CDrawScene::returnSortZItems(allBzItems, CDrawScene::EDesSort).first();
+        result = PageScene::returnSortZItems(allBzItems, PageScene::EDesSort).first();
     }
     return result;
 }
@@ -151,6 +169,23 @@ CGraphicsItem::CGraphicsItem(QGraphicsItem *parent)
     , _flipVertical(false)  // 垂直翻转
 {
     setAutoCache(false);
+    _pPrivate = new CGraphicsItem_private(this);
+}
+
+CGraphicsItem::~CGraphicsItem()
+{
+    if (_cachePixmap != nullptr) {
+        delete _cachePixmap;
+        _cachePixmap = nullptr;
+    }
+    delete _pPrivate;
+    _pPrivate = nullptr;
+}
+
+void CGraphicsItem::registerItem(const QString &classname, int classType)
+{
+    qWarning() << "classname = " << classname << "classType = " << classType;
+    CGraphicsItem_private::registedClassNameMap().insert(classType, classname);
 }
 
 void CGraphicsItem::setScene(QGraphicsScene *scene, bool calZ)
@@ -168,9 +203,9 @@ void CGraphicsItem::setScene(QGraphicsScene *scene, bool calZ)
             }
         }
     } else {
-        CDrawScene *pNewScene = qobject_cast<CDrawScene *>(scene);
+        PageScene *pNewScene = qobject_cast<PageScene *>(scene);
 
-        CDrawScene *pScene = this->drawScene();
+        PageScene *pScene = this->drawScene();
         if (pScene == pNewScene)
             return;
 
@@ -183,22 +218,60 @@ void CGraphicsItem::setScene(QGraphicsScene *scene, bool calZ)
     }
 }
 
-CGraphicsView *CGraphicsItem::curView() const
+DrawAttribution::SAttrisList CGraphicsItem::attributions()
 {
-    CGraphicsView *parentView = nullptr;
+    DrawAttribution::SAttrisList result;
+    result << DrawAttribution::SAttri(DrawAttribution::EBrushColor, brush().color())
+           << DrawAttribution::SAttri(DrawAttribution::EPenColor, pen().color())
+           << DrawAttribution::SAttri(DrawAttribution::EBorderWidth,  pen().width());
+    return result;
+}
+
+void CGraphicsItem::setAttributionVar(int attri, const QVariant &var, int phase)
+{
+    bool isPreview = (phase == EChangedBegin || phase == EChangedUpdate);
+    switch (attri) {
+    case DrawAttribution::EPenColor: {
+        setPenColor(var.value<QColor>(), isPreview);
+        break;
+    }
+    case DrawAttribution::EBrushColor: {
+        setBrushColor(var.value<QColor>(), isPreview);
+        break;
+    }
+    case DrawAttribution::EBorderWidth: {
+        setPenWidth(var.toInt(), isPreview);
+        break;
+    }
+    default:
+        break;
+    };
+}
+
+PageView *CGraphicsItem::curView() const
+{
+    PageView *parentView = nullptr;
     if (scene() != nullptr) {
         if (!scene()->views().isEmpty()) {
-            parentView = dynamic_cast<CGraphicsView *>(scene()->views().first());
+            parentView = dynamic_cast<PageView *>(scene()->views().first());
         }
     }
     return parentView;
 }
 
-CDrawScene *CGraphicsItem::drawScene() const
+PageScene *CGraphicsItem::drawScene() const
 {
     if (scene() == nullptr)
         return nullptr;
-    return qobject_cast<CDrawScene *>(scene());
+    return qobject_cast<PageScene *>(scene());
+}
+
+Page *CGraphicsItem::page() const
+{
+    if (curView() != nullptr)
+        return curView()->page();
+
+    return nullptr;
 }
 
 void CGraphicsItem::setPenColor(const QColor &c, bool isPreview)
@@ -281,6 +354,14 @@ QPen CGraphicsItem::paintPen(Qt::PenJoinStyle style) const
     return p;
 }
 
+QColor CGraphicsItem::systemThemeColor() const
+{
+    if (drawScene() != nullptr) {
+        return drawScene()->systemThemeColor();
+    }
+    return QColor();
+}
+
 bool CGraphicsItem::isMutiSelected() const
 {
     auto parentGp = bzGroup(false);
@@ -334,7 +415,7 @@ int CGraphicsItem::type() const
 
 bool CGraphicsItem::isBzItem() const
 {
-    return (this->type() >= RectType && this->type() <= BlurType);
+    return (this->type() > NoType && this->type() < MgrType);
 }
 
 bool CGraphicsItem::isSizeHandleExisted()
@@ -351,6 +432,30 @@ void CGraphicsItem::doFilp(CPictureItem::EFilpDirect dir)
     }
     update();
 }
+void CGraphicsItem::setRotation90(bool leftOrRight)
+{
+    QPointF center = this->boundingRect().center();
+    qreal angle = 0.0;
+
+    // 左旋转减小 右旋转增大
+    if (leftOrRight == true) {
+        angle = - 90.0;
+    } else {
+        angle = 90.0;
+    }
+
+    // 矩阵变换
+    QTransform trans;
+    trans.translate(center.x(), center.y());
+    trans.rotate(angle);
+    trans.translate(-center.x(), -center.y());
+
+    // 设置当前旋转角度
+    setDrawRotatin(angle + drawRotation());
+    setTransform(trans, true);
+
+    updateShapeRecursion();
+}
 
 void CGraphicsItem::setFilpBaseOrg(CPictureItem::EFilpDirect dir, bool b)
 {
@@ -412,12 +517,12 @@ QPainterPath CGraphicsItem::getTrulyShape() const
 
 QPixmap CGraphicsItem::getCachePixmap(bool baseOrg)
 {
-    if (curView() == nullptr)
-        return QPixmap();
+    qreal devecePixelRatio = curView() != nullptr ? curView()->devicePixelRatio() : qApp->devicePixelRatio();
 
-    QPixmap pix(boundingRectTruly().size().toSize()* curView()->devicePixelRatio());
+    QSizeF size = boundingRectTruly().size();
+    QPixmap pix((size * devecePixelRatio).toSize());
 
-    pix.setDevicePixelRatio(curView()->devicePixelRatio());
+    pix.setDevicePixelRatio(devecePixelRatio);
 
     pix.fill(QColor(0, 0, 0, 0));
 
@@ -447,6 +552,11 @@ QRectF CGraphicsItem::boundingRect() const
     return m_boundingRect;
 }
 
+QRectF CGraphicsItem::selectedBoundingRect() const
+{
+    return rect();
+}
+
 QRectF CGraphicsItem::boundingRectTruly() const
 {
     return m_boundingRectTrue;
@@ -543,6 +653,27 @@ QPointF CGraphicsItem::getCenter(CSizeHandleRect::EDirection dir)
     return center;
 }
 
+CGraphicsLayer *CGraphicsItem::layer()
+{
+    //return dynamic_cast<CGraphicsLayer *>(parentItem());
+    return _layer;
+}
+
+void CGraphicsItem::setLayer(CGraphicsLayer *layer)
+{
+    if (layer == _layer)
+        return;
+
+    if (_layer != nullptr) {
+        _layer->removeCItem(this);
+    }
+
+    if (layer != nullptr) {
+        layer->addCItem(this);
+    }
+    _layer = layer;
+}
+
 bool CGraphicsItem::isBzGroup(int *groupTp) const
 {
     bool result = (this->type() == MgrType);
@@ -555,7 +686,7 @@ bool CGraphicsItem::isBzGroup(int *groupTp) const
 CGraphicsItemGroup *CGraphicsItem::bzGroup(bool onlyNormal) const
 {
     if (onlyNormal) {
-        if (CDrawScene::isNormalGroupItem(_pGroup)) {
+        if (PageScene::isNormalGroupItem(_pGroup)) {
             return _pGroup;
         }
         return nullptr;
@@ -567,7 +698,7 @@ CGraphicsItemGroup *CGraphicsItem::bzTopGroup(bool onlyNormal) const
 {
     auto fPrecondition = [ = ](CGraphicsItemGroup * p) {
         if (!onlyNormal) {return p != nullptr;}
-        return CDrawScene::isNormalGroupItem(p);
+        return PageScene::isNormalGroupItem(p);
     };
 
     CGraphicsItemGroup *pTopGroup = bzGroup();
@@ -644,7 +775,7 @@ bool CGraphicsItem::contains(const QPointF &point) const
 bool CGraphicsItem::isPosPenetrable(const QPointF &posLocal)
 {
     if (curView() != nullptr) {
-        EDrawToolMode tool = curView()->getDrawParam()->getCurrentDrawToolMode();
+        int tool = drawScene()->page()->currentTool();
         if (tool == blur) {
             return false;
         }
@@ -728,7 +859,7 @@ void CGraphicsItem::operating(CGraphItemEvent *event)
         //刷新模糊路径特效
         if (!_blurInfos.isEmpty()) {
             QTransform trans = sclEvt->trans();
-            for (SBlurInfo &info : _blurInfos) {
+            foreach (SBlurInfo info, _blurInfos) {
                 info.blurPath = trans.map(info.blurPath);
             }
             if (!isCached())
@@ -794,7 +925,6 @@ void CGraphicsItem::operatingEnd(CGraphItemEvent *event)
         updateBlurPixmapBySelfBlurInfo();
         if (isCached()) {
             *_cachePixmap = getCachePixmap();
-            //qDebug() << "resize scale finished new cached size ==== " << _cachePixmap->size() << "device radio = " << _cachePixmap->devicePixelRatio();
         }
         break;
     }
@@ -896,7 +1026,7 @@ void CGraphicsItem::loadGraphicsUnit(const CGraphicsUnit &data)
 
 qreal CGraphicsItem::incLength()const
 {
-    qreal scal = drawScene() == nullptr ? 1.0 : drawScene()->drawView()->getScale();
+    qreal scal = (curView() == nullptr) ? 1.0 : curView()->getScale();
     return inccW / scal;
 }
 
@@ -1032,15 +1162,15 @@ void CGraphicsItem::updateShape()
         drawScene()->selectGroup()->updateBoundingRect();
 }
 
-void CGraphicsItem::setSizeHandleRectFlag(CSizeHandleRect::EDirection dir, bool flag)
-{
-    foreach (CSizeHandleRect *sizeHandleRect, m_handles) {
-        if (sizeHandleRect->dir() == dir) {
-            sizeHandleRect->setVisible(flag);
-            break;
-        }
-    }
-}
+//void CGraphicsItem::setSizeHandleRectFlag(CSizeHandleRect::EDirection dir, bool flag)
+//{
+//    foreach (CSizeHandleRect *sizeHandleRect, m_handles) {
+//        if (sizeHandleRect->dir() == dir) {
+//            sizeHandleRect->setVisible(flag);
+//            break;
+//        }
+//    }
+//}
 
 void CGraphicsItem::setState(ESelectionHandleState st)
 {
@@ -1080,6 +1210,11 @@ QVariant CGraphicsItem::itemChange(QGraphicsItem::GraphicsItemChange change, con
         if (bzGroup(true) != nullptr) {
             bzGroup(true)->markZDirty();
         }
+    } else if (QGraphicsItem::ItemSelectedHasChanged == change) {
+        bool select = value.toBool();
+        if (!select) {
+            //this->rasterToSelfLayer();
+        }
     }
     return value;
 }
@@ -1109,10 +1244,10 @@ void CGraphicsItem::endCheckIns(QPainter *painter)
 
 void CGraphicsItem::clearHandle()
 {
-    for (CSizeHandleRect *pItem : m_handles) {
-        pItem->setParentItem(nullptr);
+    foreach (CSizeHandleRect *pItem, m_handles) {
         if (pItem->scene() != nullptr) {
             pItem->scene()->removeItem(pItem);
+            delete pItem;
         }
     }
     m_handles.clear();
@@ -1144,91 +1279,91 @@ void CGraphicsItem::paintMutBoundingLine(QPainter *painter, const QStyleOptionGr
 
     //test
 #ifdef QT_DEBUG
-    QString testString;
-    if (this->isBzItem()) {
-        testString += QString("z = %1 ").arg(zValue());
-    }
-    if (this != drawScene()->selectGroup()) {
-        testString += QString("pos = (%1,%2) ").arg(sceneBoundingRect().x()).arg(sceneBoundingRect().y());
-        testString += QString("pos = (%1,%2) ").arg(rect().x()).arg(rect().y());
-    }
-    painter->setClipping(false);
-    QTextOption txtOption(bzGroup() == nullptr ? (Qt::AlignTop | Qt::AlignLeft) : (Qt::AlignTop | Qt::AlignRight));
-    txtOption.setWrapMode(QTextOption::NoWrap);
-    painter->drawText(boundingRect(), testString, txtOption);
+//    QString testString;
+//    if (this->isBzItem()) {
+//        testString += QString("z = %1 ").arg(zValue());
+//    }
+//    if (this != drawScene()->selectGroup()) {
+//        testString += QString("pos = (%1,%2) ").arg(sceneBoundingRect().x()).arg(sceneBoundingRect().y());
+//        testString += QString("pos = (%1,%2) ").arg(rect().x()).arg(rect().y());
+//    }
+//    painter->setClipping(false);
+//    QTextOption txtOption(bzGroup() == nullptr ? (Qt::AlignTop | Qt::AlignLeft) : (Qt::AlignTop | Qt::AlignRight));
+//    txtOption.setWrapMode(QTextOption::NoWrap);
+//    painter->drawText(boundingRect(), testString, txtOption);
 
 #endif
 }
 
 void CGraphicsItem::blurBegin(const QPointF &pos)
 {
-    _tempActiveBlurInfo.clear();
+//    _tempActiveBlurInfo.clear();
 
-    if (!isCached())
-        setCache(true);
+//    if (!isCached())
+//        setCache(true);
 
-    //生成当前图源下的模糊图像(有发生过变化都要重新生成模糊)
-    EBlurEffect wantedEf = this->curView()->getDrawParam()->getBlurEffect();
-    if (_blurPix[wantedEf].isNull() || _blurPix[wantedEf].size() != rect().size()) {
-        updateBlurPixmap(true);
-    }
+//    //生成当前图源下的模糊图像(有发生过变化都要重新生成模糊)
+//    EBlurEffect wantedEf = EBlurEffect(drawApp->defaultAttriVar(this->scene(), DrawAttribution::BlurPenEffect).toInt());
+//    if (_blurPix[wantedEf].isNull() || _blurPix[wantedEf].size() != rect().size()) {
+//        updateBlurPixmap(true);
+//    }
 
-    _tempActiveBlurInfo.blurEfTp = wantedEf;
+//    _tempActiveBlurInfo.blurEfTp = wantedEf;
 
-    _tempActiveBlurPath = QPainterPath();
-    QPointF beginPos = getFilpTransform().map(pos);
-    _tempActiveBlurPath.moveTo(beginPos);
+//    _tempActiveBlurPath = QPainterPath();
+//    QPointF beginPos = getFilpTransform().map(pos);
+//    _tempActiveBlurPath.moveTo(beginPos);
 }
 
 void CGraphicsItem::blurUpdate(const QPointF &pos, bool optm)
 {
-    QPointF newPos = getFilpTransform().map(pos);
-    QPointF oldPos = _tempActiveBlurPath.currentPosition();
-    _tempActiveBlurPath.lineTo(newPos);
-    QPen p;
-    p.setCapStyle(Qt::RoundCap);
-    p.setWidth(curView()->getDrawParam()->getBlurWidth());
+//    QPointF newPos = getFilpTransform().map(pos);
+//    QPointF oldPos = _tempActiveBlurPath.currentPosition();
+//    _tempActiveBlurPath.lineTo(newPos);
+//    QPen p;
+//    p.setCapStyle(Qt::RoundCap);
+//    p.setWidth(drawApp->defaultAttriVar(this->scene(), DrawAttribution::BlurPenWidth).toInt());
 
-    SBlurInfo blur;
+//    SBlurInfo blur;
 
-    if (!optm) {
-        //0.将所有的路径进行重新计算
-        _tempActiveBlurInfo.blurPath = CGraphicsItem::getGraphicsItemShapePathByOrg(_tempActiveBlurPath, p, true, 0, false);
-        blur = _tempActiveBlurInfo;
-    } else {
-        //1.获取新增的模糊路径(通过仅绘制新增的路径进行优化加速)
-        QPainterPath path;
-        path.moveTo(oldPos);
-        path.lineTo(newPos);
-        blur.blurPath = CGraphicsItem::getGraphicsItemShapePathByOrg(path, p, true, 0, false);
-        blur.blurEfTp = _tempActiveBlurInfo.blurEfTp;
-    }
+//    if (!optm) {
+//        //0.将所有的路径进行重新计算
+//        _tempActiveBlurInfo.blurPath = CGraphicsItem::getGraphicsItemShapePathByOrg(_tempActiveBlurPath, p, true, 0, false);
+//        blur = _tempActiveBlurInfo;
+//    } else {
+//        //1.获取新增的模糊路径(通过仅绘制新增的路径进行优化加速)
+//        QPainterPath path;
+//        path.moveTo(oldPos);
+//        path.lineTo(newPos);
+//        blur.blurPath = CGraphicsItem::getGraphicsItemShapePathByOrg(path, p, true, 0, false);
+//        blur.blurEfTp = _tempActiveBlurInfo.blurEfTp;
+//    }
 
-    //将模糊绘制到缓冲图上
-    QPainter paintToCachePixmap;
-    paintToCachePixmap.begin(_cachePixmap);
-    paintBlur(&paintToCachePixmap, blur, -rect().topLeft());
-    paintToCachePixmap.end();
-    update();
+//    //将模糊绘制到缓冲图上
+//    QPainter paintToCachePixmap;
+//    paintToCachePixmap.begin(_cachePixmap);
+//    paintBlur(&paintToCachePixmap, blur, -rect().topLeft());
+//    paintToCachePixmap.end();
+//    update();
 }
 
 void CGraphicsItem::blurEnd()
 {
-    if (!_tempActiveBlurPath.isEmpty()) {
-        QPen p;
-        p.setCapStyle(Qt::RoundCap);
-        p.setJoinStyle(Qt::RoundJoin);
-        p.setWidth(curView()->getDrawParam()->getBlurWidth());
-        _tempActiveBlurInfo.blurPath = CGraphicsItem::getGraphicsItemShapePathByOrg(_tempActiveBlurPath, p, true, 0, false);
-        _tempActiveBlurPath = QPainterPath();
-    }
+//    if (!_tempActiveBlurPath.isEmpty()) {
+//        QPen p;
+//        p.setCapStyle(Qt::RoundCap);
+//        p.setJoinStyle(Qt::RoundJoin);
+//        p.setWidth(drawApp->defaultAttriVar(this->scene(), DrawAttribution::BlurPenWidth).toInt());
+//        _tempActiveBlurInfo.blurPath = CGraphicsItem::getGraphicsItemShapePathByOrg(_tempActiveBlurPath, p, true, 0, false);
+//        _tempActiveBlurPath = QPainterPath();
+//    }
 
-    if (_tempActiveBlurInfo.isValid())
-        addBlur(_tempActiveBlurInfo);
+//    if (_tempActiveBlurInfo.isValid())
+//        addBlur(_tempActiveBlurInfo);
 
-    _tempActiveBlurInfo.clear();
+//    _tempActiveBlurInfo.clear();
 
-    update();
+//    update();
 }
 
 bool CGraphicsItem::isBlurActived()
@@ -1258,7 +1393,7 @@ void CGraphicsItem::updateShapeRecursion()
 void CGraphicsItem::updateBlurPixmap(bool baseOrg, EBlurEffect effetTp)
 {
     QPixmap pix = getCachePixmap(baseOrg);
-    EBlurEffect blurEfTp = effetTp != UnknowEffect ? effetTp : this->curView()->getDrawParam()->getBlurEffect();
+    EBlurEffect blurEfTp = effetTp != UnknowEffect ? effetTp : EBlurEffect(static_cast<int>(drawScene()->pageContext()->defaultAttri(DrawAttribution::EBlurAttri).value<SBLurEffect>().type));
     _blurPix[blurEfTp] = NSBlur::blurPixmap(pix, 10, blurEfTp);
     update();
 }
@@ -1274,25 +1409,25 @@ void CGraphicsItem::updateBlurPixmapBySelfBlurInfo()
     }
 }
 
-void CGraphicsItem::addBlur(const SBlurInfo &sblurInfo)
-{
-    if (_blurInfos.isEmpty()) {
-        _blurInfos.append(sblurInfo);
-    } else {
-        SBlurInfo &last = _blurInfos.last();
-        if (last.blurEfTp == sblurInfo.blurEfTp) {
-            last.blurPath.addPath(sblurInfo.blurPath);
-        } else {
-            _blurInfos.append(sblurInfo);
-        }
-    }
+//void CGraphicsItem::addBlur(const SBlurInfo &sblurInfo)
+//{
+//    if (_blurInfos.isEmpty()) {
+//        _blurInfos.append(sblurInfo);
+//    } else {
+//        SBlurInfo &last = _blurInfos.last();
+//        if (last.blurEfTp == sblurInfo.blurEfTp) {
+//            last.blurPath.addPath(sblurInfo.blurPath);
+//        } else {
+//            _blurInfos.append(sblurInfo);
+//        }
+//    }
 
-    update();
-}
+//    update();
+//}
 
 void CGraphicsItem::paintAllBlur(QPainter *painter, const QPointF &translate)
 {
-    for (auto info : _blurInfos) {
+    foreach (auto info, _blurInfos) {
         paintBlur(painter, info, translate);
     }
     paintBlur(painter, _tempActiveBlurInfo, translate);
@@ -1306,9 +1441,10 @@ void CGraphicsItem::paintBlur(QPainter *painter, const SBlurInfo &info, const QP
     painter->save();
     painter->setClipRect(rect().translated(translate), Qt::IntersectClip);
     painter->setClipPath(info.blurPath.translated(translate), Qt::IntersectClip);
+    if (_blurPix[info.blurEfTp].isNull()) {
+        updateBlurPixmap(true, info.blurEfTp);
+    }
     painter->drawPixmap(rect().topLeft() + translate, _blurPix[info.blurEfTp]);
-    //painter->setPen(QColor(0, 0, 0));
-    //painter->drawPath(info.blurPath.translated(translate));
     painter->restore();
 }
 void CGraphicsItem::initHandle()
@@ -1343,7 +1479,7 @@ void CGraphicsItem::resetCachePixmap()
     }
 }
 
-QTransform CGraphicsItem::getFilpTransform()
+QTransform CGraphicsItem::getFilpTransform() const
 {
     QPointF center = boundingRect().center();
     QTransform trans(this->isFilped(CGraphicsItem::EFilpHor) ? -1 : 1, 0, 0,
@@ -1353,8 +1489,70 @@ QTransform CGraphicsItem::getFilpTransform()
     return (QTransform::fromTranslate(-center.x(), -center.y()) * trans * QTransform::fromTranslate(center.x(), center.y()));
 }
 
+QPixmap CGraphicsItem::rasterSelf()
+{
+    QPixmap pix = getCachePixmap();
+    return pix;
+}
+
+void CGraphicsItem::rasterToSelfLayer(bool deleteSelf)
+{
+    QPixmap pix = this->rasterSelf();
+    QTransform itemToLayer = this->itemTransform(this->layer());
+    QPainter painter(&this->layer()->layerImage());
+
+    //先保证painter的坐标系是layer，而不是layer的image
+    painter.translate(-this->layer()->boundingRect().topLeft());
+
+    //设置了从item到layer的矩阵变化，那么接下来的绘制可以假装是在item上绘制
+    painter.setTransform(itemToLayer, true);
+
+    painter.drawPixmap(this->boundingRectTruly().toRect(), pix);
+
+    if (deleteSelf) {
+        auto scene = drawScene();
+        auto pItem = this;
+        QMetaObject::invokeMethod(scene, [ = ]() {
+            scene->removeCItem(pItem);
+            //delete pItem;
+        }, Qt::QueuedConnection);
+    }
+}
+
+//QPointF CGraphicsItem::mapFromDrawScene(const QPointF &posInDScene) const
+//{
+////    //sceneTransform() is local item to scene,then sceneTransform().inverted() is scene to local item
+////    auto trans = getFilpTransform() * sceneTransform().inverted()  /** getFilpTransform()*/;
+////    auto result = trans.map(posInDScene);
+////    return result;
+
+//    return getFilpTransform().map(mapFromScene(posInDScene));
+//}
+
+//QPointF CGraphicsItem::mapToDrawScene(const QPointF &posInThis) const
+//{
+////    auto trans = sceneTransform() * getFilpTransform().inverted();
+////    auto result = trans.map(posInThis);
+////    return result;
+
+//    return mapToScene(getFilpTransform().inverted().map(posInThis));
+//}
+
+void CGraphicsItem::updateViewport()
+{
+    if (curView() != nullptr) {
+        curView()->viewport()->update();
+    }
+}
+
 // Cppcheck检测函数没有使用到
 //void CGraphicsItem::drawItem(QPainter *painter, const QStyleOptionGraphicsItem *option)
 //{
 //    paint(painter, option, nullptr);
 //}
+
+
+RegistItemHelper::RegistItemHelper(const QString &itemname, int type)
+{
+    CGraphicsItem::registerItem(itemname, type);
+}
