@@ -25,23 +25,45 @@
 #include "globaldefine.h"
 #include "sitemdata.h"
 #include "cdrawparamsigleton.h"
+#include "cdrawtoolmanagersigleton.h"
+#include "idrawtool.h"
+#include "dyncreatobject.h"
 
 #include <QAbstractGraphicsShapeItem>
 #include <QPainterPath>
 #include <QCursor>
 #include <QStyleOptionGraphicsItem>
 
-class CGraphicsView;
+class PageView;
 
-class CDrawScene;
+class PageScene;
 
 class CGraphicsItemGroup;
+class CGraphicsLayer;
 
 class CGraphItemEvent;
 class CGraphItemMoveEvent;
 class CGraphItemScalEvent;
 class CGraphItemRotEvent;
 
+class RegistItemHelper
+{
+public:
+    RegistItemHelper(const QString &itemname, int type);
+};
+
+#define REGISTITEM_DECLARE(classname,type)\
+    REGISTCLASS_DECLARE(classname)\
+    class classname##RegistItemHelper1{static RegistItemHelper helper;};
+
+#define REGISTITEM_DEFINE(classname,type)\
+    REGISTCLASS_DEFINE(classname)\
+    RegistItemHelper classname##RegistItemHelper1::helper = RegistItemHelper(#classname,type);
+
+#define REGISTITEMCLASS(classname,type)\
+    REGISTITEM_DECLARE(classname,type) \
+    REGISTITEM_DEFINE(classname,type)
+
 class CGraphicsItem : public QAbstractGraphicsShapeItem
 {
 public:
@@ -52,6 +74,11 @@ public:
      */
     explicit CGraphicsItem(QGraphicsItem *parent);
 
+    ~CGraphicsItem() override;
+
+    static void registerItem(const QString &classname, int classType);
+    static CGraphicsItem *creatItemInstance(int itemType, const CGraphicsUnit &data = CGraphicsUnit());
+
     /**
      * @brief setScene 设置图元所处的场景
      * @param scene 场景指针
@@ -59,19 +86,26 @@ public:
     void setScene(QGraphicsScene *scene, bool calZ = true);
 
 
+    virtual DrawAttribution::SAttrisList attributions();
+    virtual void  setAttributionVar(int attri, const QVariant &var, int phase);
+
+
     enum {Type = UserType};
 
     /**
      * @brief curView 返回当前所处的view
      * @return
      */
-    CGraphicsView *curView()const;
+    PageView *curView()const;
 
     /**
      * @brief drawScene 返回当前所处的scene
      * @return
      */
-    CDrawScene *drawScene() const ;
+    PageScene *drawScene() const ;
+
+
+    Page *page()const;
 
     /**
      * @brief setPenColor 设置线颜色
@@ -103,6 +137,9 @@ public:
      */
     QPen paintPen(Qt::PenJoinStyle style = Qt::/*RoundJoin*/MiterJoin) const;
 
+
+    QColor systemThemeColor()const;
+
     /**
      * @brief rect 基于一个矩形范围的图元，所以必须实现该虚函数
      */
@@ -114,6 +151,10 @@ public:
      */
     QRectF boundingRect() const override;
 
+
+
+    virtual QRectF selectedBoundingRect() const ;
+
     /**
      * @brief boundingRectTruly 自身坐标系的真实显示的包围矩形
      * @return
@@ -161,6 +202,13 @@ public:
      */
     virtual QPointF getCenter(CSizeHandleRect::EDirection dir);
 
+    /**
+     * @brief layer 返回图元所处的图层
+     */
+    CGraphicsLayer *layer();
+
+    void setLayer(CGraphicsLayer *layer);
+
     /**
      * @brief isBzGroup 是否是一个组合图元
      * @param 如果返回值为true groupTp才有意义,返回具体的组合的类型
@@ -233,7 +281,7 @@ public:
      * @brief doFilp 在当前基础上翻转一下
      * @return
      */
-    void doFilp(EFilpDirect dir = EFilpHor);
+    virtual void doFilp(EFilpDirect dir = EFilpHor);
 
     /**
      * @brief setFilpBaseOrg 设置在初始图像上是否翻转,否则和原图一致
@@ -245,6 +293,10 @@ public:
      */
     bool isFilped(EFilpDirect dir) const;
 
+    /**
+     * @brief setRotation90
+     */
+    void setRotation90(bool leftOrRight);
 
     /**
      * @brief hitTest 碰撞检测，用来检测鼠标在图元的哪个点位上
@@ -334,7 +386,7 @@ public:
      * @param dir 方向
      * @param flag true: 显示  false:不显示
      */
-    void setSizeHandleRectFlag(CSizeHandleRect::EDirection dir, bool flag);
+    //void setSizeHandleRectFlag(CSizeHandleRect::EDirection dir, bool flag);
 
     /**
      * @brief getGraphicsItemShapePathByOrg 根据画笔属性，把图元形状转为路径   此函数为Qt源码中自带的
@@ -350,9 +402,6 @@ public:
                                                       const qreal incW = 0,
                                                       bool doSimplified = true);
 
-
-    static CGraphicsItem *creatItemInstance(int itemType, const CGraphicsUnit &data = CGraphicsUnit());
-
     template<typename T = QGraphicsItem *, typename K = CGraphicsItem *>
     static QList<T> returnList(const QList<K> &list)
     {
@@ -408,7 +457,7 @@ public:
     /**
      * @brief getFilpTransform 获取到翻转的转换矩阵
      */
-    QTransform getFilpTransform();
+    QTransform getFilpTransform() const;
 
 // Cppcheck检测函数没有使用到
 //    /**
@@ -416,6 +465,16 @@ public:
 //     */
 //    void drawItem(QPainter *painter, const QStyleOptionGraphicsItem *option);
 
+
+    QPixmap rasterSelf();
+
+    virtual void rasterToSelfLayer(bool deleteSelf = true);
+
+    //QPointF mapFromDrawScene(const QPointF &posInDScene) const;
+    //QPointF mapToDrawScene(const QPointF &posInThis) const;
+
+    void updateViewport();
+
 protected:
     /**
      * @brief loadHeadData 加载通用数据
@@ -577,28 +636,32 @@ public:
      * @brief isBlurEnable 图元是否支持模糊
      */
     virtual bool isBlurEnable()const {return false;}
+    /**
+     * @brief isBlurEnable 图元是否支持模糊
+     */
+    virtual bool isEraserEnable()const {return false;}
     /**
      * @brief blurBegin 模糊操作开始
      * @param pos 当前点(图元自身坐标系)
      */
-    void blurBegin(const QPointF &pos);
+    virtual void blurBegin(const QPointF &pos);
 
     /**
      * @brief blurUpdate 模糊操作刷新
      * @param pos 当前点(图元自身坐标系)
      * @param optm 是否进行额外优化,如果是那么在模糊结束后,记得调用一下resetCachePixmap
      */
-    void blurUpdate(const QPointF &pos, bool optm = false);
+    virtual void blurUpdate(const QPointF &pos, bool optm = false);
 
     /**
      * @brief blurEnd 模糊操作结束
      */
-    void blurEnd();
+    virtual void blurEnd();
 
     /**
      * @brief isBlurActived 模糊操作正在进行中
      */
-    bool isBlurActived();
+    virtual bool isBlurActived();
 
     /**
      * @brief setDrawRotatin 设置图元的旋转角度(仅仅是设置旋转值,不会引起图元的旋转)
@@ -631,7 +694,7 @@ protected:
     /**
      * @brief addBlur 添加一个模糊路径
      */
-    void addBlur(const SBlurInfo &sblurInfo);
+    //void addBlur(const SBlurInfo &sblurInfo);
 
     /**
      * @brief paintAllBlur 绘制所有的模糊
@@ -657,8 +720,11 @@ protected:
     bool _flipVertical   = false;   // 垂直翻转
 
     qreal            _roteAgnel = 0;         //图元的旋转角度
+
+
+    CGraphicsLayer *_layer = nullptr;
+
+    DECLAREPRIVATECLASS(CGraphicsItem)
 };
 
-//Q_DECLARE_METATYPE(CGraphicsItem::SBlurInfo);
-
 #endif // CGRAPHICSITEM_H
