@@ -24,6 +24,7 @@
 #include "frame/cgraphicsview.h"
 #include "cgraphicsitemevent.h"
 #include "cgraphicsitemselectedmgr.h"
+#include "cattributeitemwidget.h"
 
 #include <DSvgRenderer>
 
@@ -36,10 +37,9 @@
 
 DTK_USE_NAMESPACE
 
+REGISTITEMCLASS(CGraphicsLineItem, LineType)
 CGraphicsLineItem::CGraphicsLineItem(QGraphicsItem *parent)
     : CGraphicsItem(parent)
-    , m_startType(CManageViewSigleton::GetInstance()->getCurView()->getDrawParam()->getLineStartType())
-    , m_endType(CManageViewSigleton::GetInstance()->getCurView()->getDrawParam()->getLineEndType())
 {
     initLine();
 }
@@ -47,16 +47,12 @@ CGraphicsLineItem::CGraphicsLineItem(QGraphicsItem *parent)
 CGraphicsLineItem::CGraphicsLineItem(const QLineF &line, QGraphicsItem *parent)
     : CGraphicsItem(parent)
     , m_line(line)
-    , m_startType(CManageViewSigleton::GetInstance()->getCurView()->getDrawParam()->getLineStartType())
-    , m_endType(CManageViewSigleton::GetInstance()->getCurView()->getDrawParam()->getLineEndType())
 {
     initLine();
 }
 
 CGraphicsLineItem::CGraphicsLineItem(const QPointF &p1, const QPointF &p2, QGraphicsItem *parent)
     : CGraphicsItem(parent)
-    , m_startType(CManageViewSigleton::GetInstance()->getCurView()->getDrawParam()->getLineStartType())
-    , m_endType(CManageViewSigleton::GetInstance()->getCurView()->getDrawParam()->getLineEndType())
 {
     setLine(p1.x(), p1.y(), p2.x(), p2.y(), true);
     initLine();
@@ -65,15 +61,49 @@ CGraphicsLineItem::CGraphicsLineItem(const QPointF &p1, const QPointF &p2, QGrap
 CGraphicsLineItem::CGraphicsLineItem(qreal x1, qreal y1, qreal x2, qreal y2, QGraphicsItem *parent)
     : CGraphicsItem(parent)
     , m_line(x1, y1, x2, y2)
-    , m_startType(CManageViewSigleton::GetInstance()->getCurView()->getDrawParam()->getLineStartType())
-    , m_endType(CManageViewSigleton::GetInstance()->getCurView()->getDrawParam()->getLineEndType())
 {
     initLine();
 }
 
 CGraphicsLineItem::~CGraphicsLineItem()
 {
+//    m_handles.clear();
+}
 
+DrawAttribution::SAttrisList CGraphicsLineItem::attributions()
+{
+    DrawAttribution::SAttrisList result;
+    result << DrawAttribution::SAttri(DrawAttribution::EPenColor, pen().color())
+           << DrawAttribution::SAttri(DrawAttribution::EBorderWidth,  pen().width())
+           << DrawAttribution::SAttri(1775)
+           << DrawAttribution::SAttri(DrawAttribution::EStreakBeginStyle, getLineStartType())
+           << DrawAttribution::SAttri(DrawAttribution::EStreakEndStyle,  getLineEndType());
+    return result;
+}
+
+void CGraphicsLineItem::setAttributionVar(int attri, const QVariant &var, int phase)
+{
+    bool isPreview = (phase == EChangedBegin || phase == EChangedUpdate);
+    switch (attri) {
+    case DrawAttribution::EPenColor: {
+        setPenColor(var.value<QColor>(), isPreview);
+        break;
+    }
+    case DrawAttribution::EBorderWidth: {
+        setPenWidth(var.toInt(), isPreview);
+        break;
+    }
+    case DrawAttribution::EStreakBeginStyle: {
+        setLineStartType(ELineType(var.toInt()));
+        break;
+    }
+    case DrawAttribution::EStreakEndStyle: {
+        setLineEndType(ELineType(var.toInt()));
+        break;
+    }
+    default:
+        break;
+    };
 }
 
 int CGraphicsLineItem::type() const
@@ -295,6 +325,8 @@ void CGraphicsLineItem::paint(QPainter *painter, const QStyleOptionGraphicsItem
     Q_UNUSED(option)
     Q_UNUSED(widget)
 
+    //qWarning() << "bounding ========== " << this->boundingRect();
+
     updateHandlesGeometry();
 
     painter->setRenderHint(QPainter::Antialiasing, true);
@@ -346,6 +378,9 @@ void CGraphicsLineItem::initLine()
 {
     //CGraphicsItem::initHandle();
 
+    leftTop = new CSizeHandleRect(this, CSizeHandleRect::LeftTop);
+    rightBottom = new CSizeHandleRect(this, CSizeHandleRect::RightBottom);
+
     updateHandlesGeometry();
 
     this->setFlag(QGraphicsItem::ItemIsMovable, true);
@@ -362,8 +397,8 @@ void CGraphicsLineItem::initHandle()
 
     m_handles.reserve(CSizeHandleRect::None);
 
-    m_handles.push_back(new CSizeHandleRect(this, CSizeHandleRect::LeftTop));
-    m_handles.push_back(new CSizeHandleRect(this, CSizeHandleRect::RightBottom));
+    m_handles.push_back(leftTop);
+    m_handles.push_back(rightBottom);
 
     updateHandlesGeometry();
 
@@ -563,8 +598,8 @@ void CGraphicsLineItem::calcVertexes()
     updateHandlesGeometry();
 
     // 更新画布区域
-    if (scene() != nullptr)
-        scene()->views().first()->viewport()->update();
+    if (curView() != nullptr)
+        curView()->viewport()->update();
 }
 
 QPainterPath CGraphicsLineItem::getHighLightPath()
@@ -580,8 +615,8 @@ QPainterPath CGraphicsLineItem::getSelfOrgShape() const
 {
     QPainterPath path;
 
-    if (this->curView() == nullptr)
-        return path;
+//    if (this->curView() == nullptr)
+//        return path;
 
     if (m_line == QLineF())
         return path;
