@@ -24,6 +24,7 @@
 #include "frame/cviewmanagement.h"
 #include "frame/cgraphicsview.h"
 #include "cgraphicsitemevent.h"
+#include "cattributeitemwidget.h"
 
 #include <QPen>
 #include <QPainter>
@@ -39,6 +40,7 @@ const QSizeF minRectSize(10, 10);
 // 最小斜边长
 #define MinimumHypotenuseLenth 10
 
+//REGISTITEMCLASS(CGraphicsPenItem, PenType)
 CGraphicsPenItem::CGraphicsPenItem(QGraphicsItem *parent)
     : CGraphicsItem(parent)
     , m_isShiftPress(false)
@@ -78,6 +80,42 @@ CGraphicsPenItem::~CGraphicsPenItem()
 
 }
 
+DrawAttribution::SAttrisList CGraphicsPenItem::attributions()
+{
+    DrawAttribution::SAttrisList result;
+    result << DrawAttribution::SAttri(DrawAttribution::EPenColor, pen().color())
+           << DrawAttribution::SAttri(DrawAttribution::EPenWidth,  pen().width())
+           << DrawAttribution::SAttri(1775)
+           << DrawAttribution::SAttri(DrawAttribution::EStreakBeginStyle, getPenStartType())
+           << DrawAttribution::SAttri(DrawAttribution::EStreakEndStyle,  getPenEndType());
+    return result;
+}
+
+void CGraphicsPenItem::setAttributionVar(int attri, const QVariant &var, int phase)
+{
+    bool isPreview = (phase == EChangedBegin || phase == EChangedUpdate);
+    switch (attri) {
+    case DrawAttribution::EPenColor: {
+        setPenColor(var.value<QColor>(), isPreview);
+        break;
+    }
+    case DrawAttribution::EPenWidth: {
+        setPenWidth(var.toInt(), isPreview);
+        break;
+    }
+    case DrawAttribution::EStreakBeginStyle: {
+        setPenStartType(ELineType(var.toInt()));
+        break;
+    }
+    case DrawAttribution::EStreakEndStyle: {
+        setPenEndType(ELineType(var.toInt()));
+        break;
+    }
+    default:
+        break;
+    };
+}
+
 int CGraphicsPenItem::type() const
 {
     return PenType;
@@ -582,11 +620,11 @@ QPainterPath CGraphicsPenItem::getPenEndpath() const
     return m_endPath;
 }
 
-void CGraphicsPenItem::setPath(const QPainterPath &path)
-{
-    m_path = path;
-    updateShape();
-}
+//void CGraphicsPenItem::setPath(const QPainterPath &path)
+//{
+//    m_path = path;
+//    updateShape();
+//}
 
 void CGraphicsPenItem::setDrawFlag(bool flag)
 {
@@ -624,6 +662,8 @@ QPainterPath CGraphicsPenItem::getHighLightPath()
 //    path.addPath(m_startPath);
 //    path.addPath(m_endPath);
 //    return path;
+    if (bzGroup() != nullptr)
+        return QPainterPath();
     return CGraphicsItem::getHighLightPath();
 }
 
