@@ -19,14 +19,30 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "cdrawtoolmanagersigleton.h"
+#include "idrawtool.h"
+#include <QDebug>
+#include "cdrawtoolfactory.h"
 
 CDrawToolManagerSigleton *CDrawToolManagerSigleton::m_pInstance = nullptr;
 
 CDrawToolManagerSigleton::CDrawToolManagerSigleton()
 {
-    m_hashDrawTool.clear();
+    //m_hashDrawTool.clear();
 }
 
+//void CDrawToolManagerSigleton::toolManagerDeconstruction()
+//{
+////    foreach (IDrawTool *tool, m_hashDrawTool.values()) {
+////        delete tool;
+////        tool = nullptr;
+////    }
+////    m_hashDrawTool.clear();
+////    if (m_pInstance) {
+////        delete m_pInstance;
+////        m_pInstance = nullptr;
+////    }
+//}
+
 CDrawToolManagerSigleton *CDrawToolManagerSigleton::GetInstance()
 {
     if (m_pInstance == nullptr) {
@@ -35,12 +51,15 @@ CDrawToolManagerSigleton *CDrawToolManagerSigleton::GetInstance()
     return m_pInstance;
 }
 
-void CDrawToolManagerSigleton::insertDrawTool(EDrawToolMode mode, IDrawTool *tool)
-{
-    m_hashDrawTool.insert(mode, tool);
-}
+//void CDrawToolManagerSigleton::insertDrawTool(EDrawToolMode mode, IDrawTool *tool)
+//{
+//    //m_hashDrawTool.insert(mode, tool);
+//    CDrawToolFactory::installTool(tool);
+//}
 
 IDrawTool *CDrawToolManagerSigleton::getDrawTool(EDrawToolMode mode) const
 {
-    return m_hashDrawTool[mode];
+    //return m_hashDrawTool[mode];
+    return CDrawToolFactory::tool(mode);
 }
+
