@@ -23,7 +23,6 @@
 #include "cgraphicsitem.h"
 #include "cgraphicslineitem.h"
 #include "cdrawparamsigleton.h"
-#include "cgraphicsrotateangleitem.h"
 #include "cgraphicstextitem.h"
 #include "cgraphicsproxywidget.h"
 #include "cgraphicscutitem.h"
@@ -32,13 +31,10 @@
 #include "cgraphicspolygonitem.h"
 #include "cgraphicspolygonalstaritem.h"
 #include "widgets/ctextedit.h"
-//#include "cgraphicsmasicoitem.h"
 #include "bzItems/cpictureitem.h"
 #include "cgraphicsitemselectedmgr.h"
-//#include "cgraphicsitemhighlight.h"
 #include "frame/cviewmanagement.h"
 #include "frame/cgraphicsview.h"
-//#include "frame/cundocommands.h"
 #include "application.h"
 #include "frame/cundoredocommand.h"
 #include "cgraphicsitemevent.h"
@@ -57,11 +53,23 @@
 #include <QTextEdit>
 #include <QSvgGenerator>
 #include <QtMath>
+#include <DToolButton>
+
+#include "cattributeitemwidget.h"
+#include "cattributemanagerwgt.h"
+using namespace DrawAttribution;
 
 CSelectTool::CSelectTool()
     : IDrawTool(selection)
     , m_isItemMoving(false)
 {
+    connect(this, &CSelectTool::boardChanged, this, [ = ](DrawBoard * old, DrawBoard * cur) {
+        Q_UNUSED(old)
+        connect(cur, QOverload<Page *>::of(&DrawBoard::currentPageChanged), this, [ = ](Page * cur) {
+            Q_UNUSED(cur)
+            _hightLight = QPainterPath();
+        });
+    });
 }
 
 CSelectTool::~CSelectTool()
@@ -69,7 +77,103 @@ CSelectTool::~CSelectTool()
 
 }
 
-void CSelectTool::toolStart(IDrawTool::CDrawToolEvent *event, ITERecordInfo *pInfo)
+QCursor CSelectTool::cursor() const
+{
+    return QCursor(Qt::ArrowCursor);
+}
+
+QAbstractButton *CSelectTool::initToolButton()
+{
+    DToolButton *m_selectBtn = new DToolButton;
+    m_selectBtn->setShortcut(QKeySequence(QKeySequence(Qt::Key_V)));
+    setWgtAccesibleName(m_selectBtn, "Select tool button");
+    m_selectBtn->setToolTip(tr("Select (V)"));
+    m_selectBtn->setIconSize(QSize(48, 48));
+    m_selectBtn->setFixedSize(QSize(37, 37));
+    m_selectBtn->setCheckable(true);
+    //m_selectBtn->setVisible(false);
+
+    connect(m_selectBtn, &DToolButton::toggled, m_selectBtn, [ = ](bool b) {
+        QIcon icon       = QIcon::fromTheme("ddc_choose tools_normal");
+        QIcon activeIcon = QIcon::fromTheme("ddc_choose tools_active");
+        m_selectBtn->setIcon(b ? activeIcon : icon);
+    });
+    m_selectBtn->setIcon(QIcon::fromTheme("ddc_choose tools_normal"));
+    return m_selectBtn;
+}
+
+DrawAttribution::SAttrisList CSelectTool::attributions()
+{
+    DrawAttribution::SAttrisList result;
+    return result;
+}
+
+void CSelectTool::setAttributionVar(int attri, const QVariant &var, int phase, bool autoCmdStack)
+{
+    if (drawBoard()->currentPage() != nullptr)
+        drawBoard()->currentPage()->setAttributionVar(attri, var, phase, autoCmdStack);
+}
+
+void CSelectTool::registerAttributionWidgets()
+{
+    //注册自建的属性控件
+    //0.注册标题显示控件
+    _titleLabe = new QLabel;
+    //不需要重新排列属性控件，所以设置为0
+    _titleLabe->setProperty(AttriWidgetReWidth, 0);
+    QString defaultText = tr("Unnamed");
+    connect(drawBoard()->attributionWidget(), &CAttributeManagerWgt::updateWgt, this,
+    [ = ](QWidget * pWgt, const QVariant & var) {
+        if (pWgt == _titleLabe) {
+            QString text = var.isValid() ? var.toString() : defaultText;
+            //_titleLabe->setText(text);
+            text = _titleLabe->fontMetrics().elidedText(text, Qt::ElideRight, drawBoard()->attributionWidget()->width());
+            _titleLabe->setText(text);
+        }
+    });
+    drawBoard()->attributionWidget()->installEventFilter(this);
+    drawBoard()->attributionWidget()->installComAttributeWgt(ETitle, _titleLabe, defaultText);
+
+    //14.组合/取消组合 设置控件
+    auto group = new CGroupButtonWgt;
+
+    connect(group, &CGroupButtonWgt::buttonClicked, this, [ = ](bool doGroup, bool doUngroup) {
+        auto currentPage = this->drawBoard()->currentPage();
+        if (currentPage == nullptr)
+            return;
+
+        auto currentScene = currentPage->scene();
+        if (doGroup) {
+
+            CGraphicsItem *pBaseItem = currentScene->selectGroup()->getLogicFirst();
+            currentScene->creatGroup(QList<CGraphicsItem *>(), CGraphicsItemGroup::ENormalGroup,
+                                     true, pBaseItem, true);
+        }
+        if (doUngroup) {
+            currentScene->cancelGroup(nullptr, true);
+        }
+        drawBoard()->attributionWidget()->hideExpWindow();
+        //另外需要将焦点转移到text
+        auto pView = this->currentPage()->view();
+        pView->captureFocus();
+    });
+    connect(drawBoard()->attributionWidget(), &CAttributeManagerWgt::updateWgt, this,
+    [ = ](QWidget * pWgt, const QVariant & var) {
+        if (pWgt == group) {
+            bool canGroup = false;
+            bool canUnGroup = false;
+            QList<QVariant> bools = var.toList();
+            if (bools.count() == 2) {
+                canGroup   = bools[0].toBool();
+                canUnGroup = bools[1].toBool();
+            }
+            group->setGroupFlag(canGroup, canUnGroup);
+        }
+    });
+    drawBoard()->attributionWidget()->installComAttributeWgt(EGroupWgt, group);
+}
+
+void CSelectTool::toolStart(CDrawToolEvent *event, ITERecordInfo *pInfo)
 {
     _hightLight = QPainterPath();
 
@@ -125,7 +229,7 @@ void CSelectTool::toolStart(IDrawTool::CDrawToolEvent *event, ITERecordInfo *pIn
     }
 }
 
-void CSelectTool::toolUpdate(IDrawTool::CDrawToolEvent *event, ITERecordInfo *pInfo)
+void CSelectTool::toolUpdate(CDrawToolEvent *event, ITERecordInfo *pInfo)
 {
     event->setAccepted(false);
 
@@ -167,7 +271,7 @@ void CSelectTool::toolUpdate(IDrawTool::CDrawToolEvent *event, ITERecordInfo *pI
             QGraphicsItem *pItem = !pInfo->etcItems.isEmpty() ? pInfo->etcItems.first() : nullptr;
             CGraphicsItem *pMrItem = dynamic_cast<CGraphicsItem *>(pItem);
             processItemsRot(event, pInfo, EChangedUpdate);
-            drawApp->setApplicationCursor(pMrItem->handleNode()->getCursor());
+            event->scene()->setCursor(pMrItem->handleNode()->getCursor());
         }
         break;
     }
@@ -176,7 +280,7 @@ void CSelectTool::toolUpdate(IDrawTool::CDrawToolEvent *event, ITERecordInfo *pI
     }
 }
 
-void CSelectTool::toolFinish(IDrawTool::CDrawToolEvent *event, ITERecordInfo *pInfo)
+void CSelectTool::toolFinish(CDrawToolEvent *event, ITERecordInfo *pInfo)
 {
     bool doUndoFinish = true;
     switch (pInfo->_opeTpUpdate) {
@@ -225,6 +329,7 @@ void CSelectTool::toolFinish(IDrawTool::CDrawToolEvent *event, ITERecordInfo *pI
         QGraphicsItem *pItem = items.first();
         if (pItem != nullptr && pItem->type() == PictureType) {
             event->scene()->selectGroup()->updateAttributes();
+            event->scene()->updateAttribution();
         }
     }
 
@@ -232,13 +337,18 @@ void CSelectTool::toolFinish(IDrawTool::CDrawToolEvent *event, ITERecordInfo *pI
     if (doUndoFinish)
         CUndoRedoCommand::finishRecord();
 
+    mouseHoverEvent(event);
+
     IDrawTool::toolFinish(event, pInfo);
 }
 
-void CSelectTool::toolDoubleClikedEvent(IDrawTool::CDrawToolEvent *event, IDrawTool::ITERecordInfo *pInfo)
+void CSelectTool::toolDoubleClikedEvent(CDrawToolEvent *event, IDrawTool::ITERecordInfo *pInfo)
 {
     IDrawTool::toolDoubleClikedEvent(event, pInfo);
 
+    //修复双击形成多选
+    event->scene()->clearSelectGroup();
+
     if (pInfo->startPosTopBzItem != nullptr) {
         if (pInfo->startPosTopBzItem->type() == TextType) {
             CGraphicsTextItem *pTextItem = dynamic_cast<CGraphicsTextItem *>(pInfo->startPosTopBzItem);
@@ -252,10 +362,10 @@ void CSelectTool::toolDoubleClikedEvent(IDrawTool::CDrawToolEvent *event, IDrawT
     event->setAccepted(true);
 }
 
-int CSelectTool::decideUpdate(IDrawTool::CDrawToolEvent *event, IDrawTool::ITERecordInfo *pInfo)
+int CSelectTool::decideUpdate(CDrawToolEvent *event, IDrawTool::ITERecordInfo *pInfo)
 {
     int tpye = ENothingDo;
-    if (isActived()) {
+    if (isWorking()) {
         QGraphicsItem *pStartPosTopQtItem = event->scene()->firstItem(pInfo->_startPos,
                                                                       pInfo->startPosItems,
                                                                       true, true);
@@ -281,7 +391,7 @@ int CSelectTool::decideUpdate(IDrawTool::CDrawToolEvent *event, IDrawTool::ITERe
                         event->scene()->recordSecenInfoToCmd(CSceneUndoRedoCommand::EGroupChanged, UndoVar);
 
                         //1.复制出一样的一个组合图元(这里复制的是选择组合图元,之后应该删掉它)
-                        CGraphicsItemGroup *pNewGroup = CDrawScene::copyCreatGroup(event->scene()->selectGroup());
+                        CGraphicsItemGroup *pNewGroup = PageScene::copyCreatGroup(event->scene()->selectGroup());
 
                         QList<CGraphicsItem *> needSelectItems;
                         if (pNewGroup != nullptr) {
@@ -343,23 +453,41 @@ int CSelectTool::decideUpdate(IDrawTool::CDrawToolEvent *event, IDrawTool::ITERe
     return tpye;
 }
 
-void CSelectTool::mouseHoverEvent(IDrawTool::CDrawToolEvent *event)
+void CSelectTool::mouseHoverEvent(CDrawToolEvent *event)
 {
     //处理高亮，鼠标样式变化等问题
-    event->scene()->refreshLook(event->pos());
+    processHightLight(event);
+
+    processCursor(event);
+}
+
+void CSelectTool::leaveEvent(CDrawToolEvent *event)
+{
+    Q_UNUSED(event);
+    _hightLight = QPainterPath();
+    event->scene()->update();
 }
 
 void CSelectTool::drawMore(QPainter *painter,
                            const QRectF &rect,
-                           CDrawScene *scene)
+                           PageScene *scene)
 {
     //注意painter是在scene的坐标系
 
     Q_UNUSED(rect)
     painter->save();
-    for (auto it = _allITERecordInfo.begin();
-            it != _allITERecordInfo.end(); ++it) {
-        ITERecordInfo &info = it.value();
+
+    if (_allITERecordInfo.isEmpty()) {
+        painter->setBrush(Qt::NoBrush);
+        QColor selectColor = scene->systemThemeColor();
+        QPen p(selectColor);
+        p.setWidthF(2.0);
+        painter->setPen(p);
+        if (!_hightLight.isEmpty()) {
+            painter->drawPath(_hightLight);
+        }
+    }
+    foreach (auto info, _allITERecordInfo.values()) {
         if (info._opeTpUpdate == ERectSelect) {
             QPointF pos0 = info._startPos;
             QPointF pos1 = info._curEvent.pos();
@@ -369,7 +497,7 @@ void CSelectTool::drawMore(QPainter *painter,
 
             QPen pen;
             pen.setWidthF(0.5);
-            QBrush selectBrush = drawApp->systemThemeColor();
+            QBrush selectBrush = scene->systemThemeColor();
             QColor selectColor = selectBrush.color();
             selectColor.setAlpha(20);
             selectBrush.setColor(selectColor);
@@ -413,6 +541,19 @@ void CSelectTool::drawMore(QPainter *painter,
     painter->restore();
 }
 
+void CSelectTool::onStatusChanged(EStatus oldStatus, EStatus nowStatus)
+{
+    Q_UNUSED(oldStatus)
+    if (nowStatus == EIdle) {
+        if (drawBoard()->currentPage() != nullptr) {
+            auto w = drawBoard()->currentPage()->view()->activeProxWidget();
+            if (w != nullptr) {
+                w->clearFocus();
+            }
+        }
+    }
+}
+
 bool CSelectTool::returnToSelectTool(CDrawToolEvent *event, ITERecordInfo *pInfo)
 {
     Q_UNUSED(event)
@@ -459,7 +600,7 @@ void CSelectTool::sendToolEventToItem(CDrawToolEvent *event,
     }
 }
 
-void CSelectTool::processItemsScal(IDrawTool::CDrawToolEvent *event, IDrawTool::ITERecordInfo *info, EChangedPhase phase)
+void CSelectTool::processItemsScal(CDrawToolEvent *event, IDrawTool::ITERecordInfo *info, EChangedPhase phase)
 {
     CGraphItemScalEvent scal(CGraphItemEvent::EScal);
     scal.setEventPhase(phase);
@@ -485,7 +626,7 @@ void CSelectTool::processItemsScal(IDrawTool::CDrawToolEvent *event, IDrawTool::
 
     //分发事件
     for (auto item : info->etcItems) {
-        if (CDrawScene::isDrawItem(item) || item == event->scene()->selectGroup()) {
+        if (PageScene::isDrawItem(item) || item == event->scene()->selectGroup()) {
             CGraphicsItem *pBzItem = static_cast<CGraphicsItem *>(item);
 
             scal.setPos(pBzItem->mapFromScene(event->pos()));
@@ -507,7 +648,7 @@ void CSelectTool::processItemsScal(IDrawTool::CDrawToolEvent *event, IDrawTool::
     event->view()->viewport()->update();
 }
 
-void CSelectTool::processItemsRot(IDrawTool::CDrawToolEvent *event, IDrawTool::ITERecordInfo *info, EChangedPhase phase)
+void CSelectTool::processItemsRot(CDrawToolEvent *event, IDrawTool::ITERecordInfo *info, EChangedPhase phase)
 {
     CGraphItemRotEvent rot(CGraphItemEvent::ERot);
     rot.setEventPhase(phase);
@@ -519,7 +660,7 @@ void CSelectTool::processItemsRot(IDrawTool::CDrawToolEvent *event, IDrawTool::I
 
     //分发事件
     for (auto item : info->etcItems) {
-        if (CDrawScene::isDrawItem(item) || item == event->scene()->selectGroup()) {
+        if (PageScene::isDrawItem(item) || item == event->scene()->selectGroup()) {
             CGraphicsItem *pBzItem = static_cast<CGraphicsItem *>(item);
             rot.setPos(pBzItem->mapFromScene(event->pos()));
             rot.setOldPos(pBzItem->mapFromScene(info->_prePos));
@@ -538,6 +679,56 @@ void CSelectTool::processItemsRot(IDrawTool::CDrawToolEvent *event, IDrawTool::I
     event->view()->viewport()->update();
 }
 
+void CSelectTool::processHightLight(CDrawToolEvent *event)
+{
+    auto hlPath = QPainterPath();
+
+    if (event->view()->activeProxWidget() == nullptr) {
+        QGraphicsItem *pItem = event->scene()->firstItem(event->pos(), QList<QGraphicsItem *>(),
+                                                         true, true, true, true);
+        if (pItem != nullptr) {
+            CGraphicsItem *pBzItem = static_cast<CGraphicsItem *>(pItem);
+            hlPath = pBzItem->mapToScene(pBzItem->getHighLightPath());
+        }
+    }
+    if (hlPath != _hightLight) {
+        _hightLight = hlPath;
+        event->scene()->update();
+    }
+}
+
+void CSelectTool::processCursor(CDrawToolEvent *event)
+{
+    QPointF scenePos = event->pos();
+
+    QList<QGraphicsItem *> items = event->scene()->items(scenePos);
+
+    QGraphicsItem *pItem = event->scene()->firstItem(scenePos, items, true, true, false, false, false);
+    if (pItem != nullptr) {
+        CSizeHandleRect *pRectItem = dynamic_cast<CSizeHandleRect *>(pItem);
+        if (pRectItem != nullptr) {
+            event->scene()->setCursor(pRectItem->getCursor());
+        } else {
+            event->scene()->setCursor(pItem->cursor());
+        }
+    } else {
+        event->scene()->setCursor(cursor());
+    }
+}
+
+bool CSelectTool::eventFilter(QObject *o, QEvent *e)
+{
+    auto borad = drawBoard();
+    if (borad != nullptr && borad->currentPage() != nullptr && borad->currentPage()->currentTool() == selection) {
+        if (o == borad->attributionWidget() && (e->type() == QEvent::Resize || e->type() == QEvent::Show)) {
+            auto text = borad->currentPage()->name();
+            text = _titleLabe->fontMetrics().elidedText(text, Qt::ElideRight, borad->attributionWidget()->width());
+            _titleLabe->setText(text);
+        }
+    }
+    return IDrawTool::eventFilter(o, e);
+}
+
 void CSelectTool::processItemsMove(CDrawToolEvent *event,
                                    ITERecordInfo *info,
                                    EChangedPhase phase)
@@ -551,8 +742,8 @@ void CSelectTool::processItemsMove(CDrawToolEvent *event,
     mov._sceneBeginPos = info->_startPos;
 
     //分发事件
-    for (auto item : info->etcItems) {
-        if (CDrawScene::isDrawItem(item) || item == event->scene()->selectGroup()) {
+    foreach (auto item, info->etcItems) {
+        if (PageScene::isDrawItem(item) || item == event->scene()->selectGroup()) {
             CGraphicsItem *pBzItem = static_cast<CGraphicsItem *>(item);
             mov.setPos(pBzItem->mapFromScene(event->pos()));
             mov.setOldPos(pBzItem->mapFromScene(info->_prePos));
