@@ -26,21 +26,22 @@
 #include "application.h"
 #include "frame/cundoredocommand.h"
 #include "cgraphicsitem.h"
+#include "cdrawtoolfactory.h"
+
+
+
 #include <QDebug>
 #include <QKeyEvent>
 #include <QGraphicsSceneEvent>
 #include <QGraphicsProxyWidget>
 #include <QTimer>
+#include <QToolButton>
 
 QTimer *IDrawTool::s_timerForDoubleClike = nullptr;
 
-IDrawTool::~IDrawTool()
-{
-
-}
-
 IDrawTool::IDrawTool(EDrawToolMode mode)
-    : m_bMousePress(false)
+    : QObject(nullptr)
+    , m_bMousePress(false)
     , m_sPointPress(0, 0)
     , m_sLastPress(0, 0)
     , m_sPointRelease(0, 0)
@@ -53,15 +54,111 @@ IDrawTool::IDrawTool(EDrawToolMode mode)
     , m_RightTopCursor(QPixmap(":/theme/light/images/mouse_style/icon_drag_rightup.svg").scaled(24, 24, Qt::IgnoreAspectRatio, Qt::SmoothTransformation))
     , m_LeftRightCursor(QPixmap(":/theme/light/images/mouse_style/icon_drag_left.svg").scaled(24, 24, Qt::IgnoreAspectRatio, Qt::SmoothTransformation))
     , m_UpDownCursor(QPixmap(":/theme/light/images/mouse_style/icon_drag_up.svg").scaled(24, 24, Qt::IgnoreAspectRatio, Qt::SmoothTransformation))
+{
+    QMetaObject::invokeMethod(this, [ = ]() {
+        if (!_registedWidgets) {
+            //registerAttributionWidgets();
+            _registedWidgets = true;
+        }
+    }, Qt::QueuedConnection);
+
+
+    connect(this, &IDrawTool::statusChanged, this, [ = ](EStatus oldStatus, EStatus newStatus) {
+//        if (newStatus == EReady) {
+//            qWarning() << "status changed to EReady, tool type is: " << getDrawToolMode();
+//        }
+        onStatusChanged(oldStatus, newStatus);
+    });
+}
+
+IDrawTool::EStatus IDrawTool::status()
+{
+    return _status;
+}
+
+void IDrawTool::setEnable(bool b)
+{
+//    if (b) {
+//        if (isEnable(CURRENTVIEW)) {
+//            toolButton()->setEnabled(true);
+//            _status = EIdle;
+//        } else {
+//            qWarning() << "tool can not set to enable because virtual function \"isEnable\" return false in current view!";
+//        }
+//    } else {
+//        toolButton()->setEnabled(false);
+//        _status = EDisAbled;
+//    }
+
+    toolButton()->setEnabled(b);
+    changeStatusFlagTo(b ? EIdle : EDisAbled);
+}
+
+bool IDrawTool::activeTool()
+{
+    //return CDrawToolFactory::setCurrentTool(getDrawToolMode(), false);
+    toolButton()->setChecked(true);
+    return true;
+}
+
+QAbstractButton *IDrawTool::toolButton()
+{
+    if (_pToolButton == nullptr) {
+        _pToolButton = initToolButton();
+    }
+    return _pToolButton;
+}
+
+QCursor IDrawTool::cursor() const
+{
+    return QCursor(Qt::ArrowCursor);
+}
+
+QAbstractButton *IDrawTool::initToolButton()
+{
+    return nullptr;
+}
+
+DrawAttribution::SAttrisList IDrawTool::attributions()
+{
+    return QList<int>() << 0;
+}
+
+void IDrawTool::setAttributionVar(int attri, const QVariant &var, int phase, bool autoCmdStack)
+{
+    Q_UNUSED(attri)
+    Q_UNUSED(var)
+    Q_UNUSED(phase)
+    Q_UNUSED(autoCmdStack)
+}
+
+void IDrawTool::registerAttributionWidgets()
 {
 
 }
-void IDrawTool::mousePressEvent(QGraphicsSceneMouseEvent *event, CDrawScene *scene)
+
+void IDrawTool::onStatusChanged(EStatus oldStatus, EStatus nowStatus)
+{
+    if (currentPage() == nullptr)
+        return;
+
+    if (oldStatus == EIdle && nowStatus == EReady) {
+        if (this->currentPage()->view() != nullptr)
+            this->currentPage()->view()->drawScene()->clearSelectGroup();
+    }
+}
+
+IDrawTool::~IDrawTool()
+{
+
+}
+
+void IDrawTool::mousePressEvent(QGraphicsSceneMouseEvent *event, PageScene *scene)
 {
     //1.如果由qt将触控事件转成的鼠标事件那么不要再调用toolDoStart（因为在scene的event中已经处理过）
-    if (event->source() == Qt::MouseEventSynthesizedByQt) {
-        return scene->mouseEvent(event);
-    }
+//    if (event->source() == Qt::MouseEventSynthesizedByQt) {
+//        return scene->mouseEvent(event);
+//    }
     CDrawToolEvent::CDrawToolEvents e = CDrawToolEvent::fromQEvent(event, scene);
 
     toolDoStart(&e.first());
@@ -71,12 +168,12 @@ void IDrawTool::mousePressEvent(QGraphicsSceneMouseEvent *event, CDrawScene *sce
     }
 }
 
-void IDrawTool::mouseMoveEvent(QGraphicsSceneMouseEvent *event, CDrawScene *scene)
+void IDrawTool::mouseMoveEvent(QGraphicsSceneMouseEvent *event, PageScene *scene)
 {
     //1.如果由qt将触控事件转成的鼠标事件那么不要再调用toolDoUpdate（因为在scene的event中已经处理过）
-    if (event->source() == Qt::MouseEventSynthesizedByQt) {
-        return scene->mouseEvent(event);
-    }
+//    if (event->source() == Qt::MouseEventSynthesizedByQt) {
+//        return scene->mouseEvent(event);
+//    }
 
     CDrawToolEvent::CDrawToolEvents e = CDrawToolEvent::fromQEvent(event, scene);
 
@@ -88,12 +185,12 @@ void IDrawTool::mouseMoveEvent(QGraphicsSceneMouseEvent *event, CDrawScene *scen
     }
 }
 
-void IDrawTool::mouseReleaseEvent(QGraphicsSceneMouseEvent *event, CDrawScene *scene)
+void IDrawTool::mouseReleaseEvent(QGraphicsSceneMouseEvent *event, PageScene *scene)
 {
     //1.如果由qt将触控事件转成的鼠标事件那么不要再调用toolDoFinish（因为在scene的event中已经处理过）
-    if (event->source() == Qt::MouseEventSynthesizedByQt) {
-        return scene->mouseEvent(event);
-    }
+//    if (event->source() == Qt::MouseEventSynthesizedByQt) {
+//        return scene->mouseEvent(event);
+//    }
 
     CDrawToolEvent::CDrawToolEvents e = CDrawToolEvent::fromQEvent(event, scene);
 
@@ -104,7 +201,7 @@ void IDrawTool::mouseReleaseEvent(QGraphicsSceneMouseEvent *event, CDrawScene *s
         scene->mouseEvent(event);
     }
 }
-void IDrawTool::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event, CDrawScene *scene)
+void IDrawTool::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event, PageScene *scene)
 {
     //1.如果由qt将触控事件转成的鼠标事件那么不要再调用（因为在scene的event中已经处理过）
     if (event->source() == Qt::MouseEventSynthesizedByQt) {
@@ -120,7 +217,17 @@ void IDrawTool::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event, CDrawScen
     }
 }
 
-bool IDrawTool::isActived()
+void IDrawTool::enterEvent(CDrawToolEvent *event)
+{
+    Q_UNUSED(event)
+}
+
+void IDrawTool::leaveEvent(CDrawToolEvent *event)
+{
+    Q_UNUSED(event)
+}
+
+bool IDrawTool::isWorking()
 {
     return !_allITERecordInfo.isEmpty();
 }
@@ -128,26 +235,51 @@ bool IDrawTool::isActived()
 void IDrawTool::interrupt()
 {
     clearITE();
+    if (status() == EWorking)
+        changeStatusFlagTo(EReady);
 }
 
-//int IDrawTool::activedType()
+DrawBoard *IDrawTool::drawBoard() const
+{
+    return _drawBoard;
+}
+
+void IDrawTool::setDrawBoard(DrawBoard *board)
+{
+    if (board != _drawBoard) {
+        auto old = _drawBoard;
+        _drawBoard = board;
+        emit boardChanged(old, _drawBoard);
+    }
+}
+
+Page *IDrawTool::currentPage() const
+{
+    if (drawBoard() != nullptr) {
+        return drawBoard()->currentPage();
+    }
+    return nullptr;
+}
+
+void IDrawTool::setTouchSensitiveRadius(int sensitiveRadius)
+{
+    _touchSensitiveRadius = sensitiveRadius;
+}
+
+//int IDrawTool::touchSensitiveRadius() const
 //{
-//    if (!_allITERecordInfo.isEmpty()) {
-//        return _allITERecordInfo.last()._opeTpUpdate;
-//    }
-//    return -1;
+//    return _touchSensitiveRadius;
 //}
 
-void IDrawTool::toolDoStart(IDrawTool::CDrawToolEvent *event)
+void IDrawTool::toolDoStart(CDrawToolEvent *event)
 {
-    event->scene()->clearHighlight();
+    //event->scene()->clearHighlight();
     if (event->mouseButtons() == Qt::LeftButton || event->eventType() == CDrawToolEvent::ETouchEvent) {
 
-        //m_bMousePress = true;
         if (dueTouchDoubleClickedStart(event)) {
             return;
         }
-        int incW = drawApp->touchFeelingEnhanceValue();
+        int incW = drawBoard()->touchFeelingEnhanceValue();
         ITERecordInfo info;
 
         info._prePos       = event->pos();
@@ -156,7 +288,7 @@ void IDrawTool::toolDoStart(IDrawTool::CDrawToolEvent *event)
                              event->scene()->items(QRectF(event->pos() - QPoint(incW, incW), QSizeF(2 * incW, 2 * incW))) :
                              event->scene()->items(event->pos());
         info.startPosTopBzItem = event->scene()->topBzItem(event->pos(), true,
-                                                           event->eventType() == CDrawToolEvent::ETouchEvent ? drawApp->touchFeelingEnhanceValue() : 0);
+                                                           event->eventType() == CDrawToolEvent::ETouchEvent ? drawBoard()->touchFeelingEnhanceValue() : 0);
         info._isvaild  = true;
         info._curEvent = *event;
         info._startEvent = *event;
@@ -179,6 +311,14 @@ void IDrawTool::toolDoStart(IDrawTool::CDrawToolEvent *event)
                 _allITERecordInfo.insert(event->uuid(), info);
         } else {
             info.businessItem  = creatItem(event, &info);
+            //设置默认属性
+            if (info.businessItem != nullptr) {
+                auto attrisList = this->attributions();
+                foreach (auto attri, attrisList) {
+                    info.businessItem->setAttributionVar(attri.attri, attri.var, EChanged);
+                }
+            }
+
             info.eventLife     = ENormal;
             qDebug() << "toolDoStart rInfo.eventLife = ENormal id " << event->uuid();
 
@@ -198,7 +338,7 @@ void IDrawTool::toolDoStart(IDrawTool::CDrawToolEvent *event)
     }
 }
 
-void IDrawTool::toolDoUpdate(IDrawTool::CDrawToolEvent *event)
+void IDrawTool::toolDoUpdate(CDrawToolEvent *event)
 {
     if (!_allITERecordInfo.isEmpty()) {
         auto it = _allITERecordInfo.find(event->uuid());
@@ -212,7 +352,7 @@ void IDrawTool::toolDoUpdate(IDrawTool::CDrawToolEvent *event)
                 int constDis = minMoveUpdateDistance();
 
                 QPointF offset = event->pos(CDrawToolEvent::EViewportPos) -
-                                 rInfo._startEvent.pos(CDrawToolEvent::EViewportPos)/*event->view()->mapFromScene(rInfo._startPos)*/;
+                                 rInfo._startEvent.pos(CDrawToolEvent::EViewportPos);
                 int curDis = qRound(offset.manhattanLength());
                 rInfo._moved = (curDis >= constDis);
             }
@@ -231,8 +371,15 @@ void IDrawTool::toolDoUpdate(IDrawTool::CDrawToolEvent *event)
                             rInfo.haveDecidedOperateType = true;
                         }
                     } else {
-                        QRectF rectf(event->view()->mapFromScene(rInfo._startPos) - QPointF(10, 10), QSizeF(20, 20));
-                        if (!rectf.contains(event->pos(CDrawToolEvent::EViewportPos))) {
+                        bool doDecide = true;
+                        bool isTouch = (event->eventType() == CDrawToolEvent::ETouchEvent);
+                        if (isTouch) {
+                            if (_touchSensitiveRadius != 0) {
+                                QRectF rectf(event->view()->mapFromScene(rInfo._startPos) - QPointF(_touchSensitiveRadius, _touchSensitiveRadius), QSizeF(2 * _touchSensitiveRadius, 2 * _touchSensitiveRadius));
+                                doDecide = !rectf.contains(event->pos(CDrawToolEvent::EViewportPos));
+                            }
+                        }
+                        if (doDecide) {
                             QTime *elTi = rInfo.getTimeHandle();
                             rInfo._elapsedToUpdate = (elTi == nullptr ? -1 : elTi->elapsed());
                             rInfo._opeTpUpdate = decideUpdate(event, &rInfo);
@@ -242,14 +389,19 @@ void IDrawTool::toolDoUpdate(IDrawTool::CDrawToolEvent *event)
                             }
 
                             rInfo.haveDecidedOperateType = true;
+                        } else {
+                            event->setPosXAccepted(false);
+                            event->setPosYAccepted(false);
                         }
                     }
                 }
                 //2.执行操作
                 if (rInfo.haveDecidedOperateType) {
                     if (rInfo._opeTpUpdate == EToolCreatItemMove) {
+                        changeStatusFlagTo(EWorking);
                         toolCreatItemUpdate(event, &rInfo);
                     } else if (rInfo._opeTpUpdate > EToolDoNothing) {
+                        changeStatusFlagTo(EWorking);
                         toolUpdate(event, &rInfo);
                     }
                 }
@@ -265,9 +417,8 @@ void IDrawTool::toolDoUpdate(IDrawTool::CDrawToolEvent *event)
     }
 }
 
-void IDrawTool::toolDoFinish(IDrawTool::CDrawToolEvent *event)
+void IDrawTool::toolDoFinish(CDrawToolEvent *event)
 {
-    bool updateCursor = true;
     qDebug() << "toolDoFinish ==== " << event->uuid();
     if (!_allITERecordInfo.isEmpty()) {
         auto it = _allITERecordInfo.find(event->uuid());
@@ -290,19 +441,23 @@ void IDrawTool::toolDoFinish(IDrawTool::CDrawToolEvent *event)
                     pCreatedItem = rInfo.businessItem;
                     if (rInfo.businessItem != nullptr) {
                         if (rInfo.businessItem->scene() == event->scene()) {
-                            //qWarning() << "zzzzzzzzz = " << rInfo.businessItem->zValue();
-                            if (autoSupUndoForCreatItem())
-                                CCmdBlock block(event->scene(), CSceneUndoRedoCommand::EItemAdded, rInfo.businessItem);
-                            //event->scene()->selectItem(rInfo.businessItem);
+                            if (rInfo.businessItem->layer() != nullptr && rasterItemToLayer(event, &rInfo)) {
+                                rInfo.businessItem->rasterToSelfLayer(true);
+                                rInfo.businessItem = nullptr;
+                                pCreatedItem = nullptr;
+                            } else {
+                                if (autoSupUndoForCreatItem())
+                                    CCmdBlock block(event->scene(), CSceneUndoRedoCommand::EItemAdded, rInfo.businessItem);
+                            }
                         }
                     }
-                } else { /*if (rInfo._opeTpUpdate > EToolDoNothing)*/
+                } else {
                     if (rInfo._opeTpUpdate > EToolDoNothing)
                         sendToolEventToItem(event, &rInfo, EChangedFinished);
                     toolFinish(event, &rInfo);
                 }
                 // 保证恢复到正常绘制
-                if (event->view()->isCacheEnabled()) {
+                if (event->view()->isCacheEnabled() && _allITERecordInfo.count() == 1) {
                     event->view()->setCacheEnable(false);
                 }
             }
@@ -310,34 +465,57 @@ void IDrawTool::toolDoFinish(IDrawTool::CDrawToolEvent *event)
             qDebug() << "finished uuid ===== " << event->uuid() << "allITERecordInfo count = " << _allITERecordInfo.count();
 
             //2.是否要回到select工具模式下去(多个触控时即将为空才判断)
-            if (_allITERecordInfo.count() == 1 && returnToSelectTool(event, &rInfo)) {
-                if (pCreatedItem != nullptr) {
-                    event->scene()->selectItem(pCreatedItem);
-                    event->scene()->setFocusItem(pCreatedItem);
+            if (_allITERecordInfo.count() == 1) {
+
+                if (returnToSelectTool(event, &rInfo)) {
+                    if (pCreatedItem != nullptr) {
+                        event->scene()->selectItem(pCreatedItem);
+                        event->scene()->setFocusItem(pCreatedItem);
+                    }
+                    changeStatusFlagTo(EReady);
+                    drawBoard()->setCurrentTool(selection);
+                } else {
+                    changeStatusFlagTo(EReady);
                 }
-                setViewToSelectionTool();
-            } else {
-                //不回到select工具时不用刷新鼠标样式（维持工具的鼠标样式方便提示用户可继续绘制）
-                updateCursor = false;
             }
             _allITERecordInfo.erase(it);
         }
     } else {
         event->setAccepted(false);
     }
-    if (event->eventType() == CDrawToolEvent::EMouseEvent && updateCursor)
-        event->scene()->refreshLook(event->pos());
 
     event->view()->setFocus();
 }
 
-bool IDrawTool::isEnable(CGraphicsView *pView)
+bool IDrawTool::isEnable(PageView *pView)
 {
     Q_UNUSED(pView)
     return true;
 }
 
-bool IDrawTool::dueTouchDoubleClickedStart(IDrawTool::CDrawToolEvent *event)
+void IDrawTool::refresh()
+{
+    auto curView = currentPage() != nullptr ? currentPage()->view() : nullptr;
+    if (curView != nullptr) {
+        auto viewPos  = curView->viewport()->mapFromGlobal(QCursor::pos());
+        auto scenePos = curView->mapToScene(viewPos);
+        curView->viewport()->setCursor(cursor());
+        if (qApp->activePopupWidget() != nullptr) {
+            CDrawToolEvent event(QPointF(), QPointF(), QPointF(), curView->drawScene());
+            mouseHoverEvent(&event);
+            return;
+        }
+        CDrawToolEvent event(viewPos, scenePos, QCursor::pos(), curView->drawScene());
+        mouseHoverEvent(&event);
+    }
+}
+
+DrawAttribution::SAttri IDrawTool::defaultAttriVar(int attri) const
+{
+    return DrawAttribution::SAttri(attri, drawBoard()->defaultAttriVar(drawBoard()->currentPage(), attri));
+}
+
+bool IDrawTool::dueTouchDoubleClickedStart(CDrawToolEvent *event)
 {
     static const int intervalMs = 250;
     if (event->eventType() == CDrawToolEvent::ETouchEvent) {
@@ -404,7 +582,7 @@ bool IDrawTool::dueTouchDoubleClickedStart(IDrawTool::CDrawToolEvent *event)
     return false;
 }
 
-void IDrawTool::toolStart(IDrawTool::CDrawToolEvent *event, ITERecordInfo *pInfo)
+void IDrawTool::toolStart(CDrawToolEvent *event, ITERecordInfo *pInfo)
 {
     Q_UNUSED(pInfo)
 
@@ -412,7 +590,7 @@ void IDrawTool::toolStart(IDrawTool::CDrawToolEvent *event, ITERecordInfo *pInfo
     event->setAccepted(true);
 }
 
-void IDrawTool::toolUpdate(IDrawTool::CDrawToolEvent *event, ITERecordInfo *pInfo)
+void IDrawTool::toolUpdate(CDrawToolEvent *event, ITERecordInfo *pInfo)
 {
     Q_UNUSED(pInfo)
 
@@ -420,7 +598,7 @@ void IDrawTool::toolUpdate(IDrawTool::CDrawToolEvent *event, ITERecordInfo *pInf
     event->setAccepted(true);
 }
 
-void IDrawTool::toolFinish(IDrawTool::CDrawToolEvent *event, ITERecordInfo *pInfo)
+void IDrawTool::toolFinish(CDrawToolEvent *event, ITERecordInfo *pInfo)
 {
     Q_UNUSED(pInfo)
 
@@ -430,7 +608,7 @@ void IDrawTool::toolFinish(IDrawTool::CDrawToolEvent *event, ITERecordInfo *pInf
     event->scene()->update();
 }
 
-int IDrawTool::decideUpdate(IDrawTool::CDrawToolEvent *event, IDrawTool::ITERecordInfo *pInfo)
+int IDrawTool::decideUpdate(CDrawToolEvent *event, IDrawTool::ITERecordInfo *pInfo)
 {
     Q_UNUSED(event)
     Q_UNUSED(pInfo)
@@ -439,7 +617,7 @@ int IDrawTool::decideUpdate(IDrawTool::CDrawToolEvent *event, IDrawTool::ITEReco
     return -2;
 }
 
-void IDrawTool::toolCreatItemStart(IDrawTool::CDrawToolEvent *event, IDrawTool::ITERecordInfo *pInfo)
+void IDrawTool::toolCreatItemStart(CDrawToolEvent *event, IDrawTool::ITERecordInfo *pInfo)
 {
     Q_UNUSED(pInfo)
 
@@ -449,7 +627,7 @@ void IDrawTool::toolCreatItemStart(IDrawTool::CDrawToolEvent *event, IDrawTool::
     event->scene()->update();
 }
 
-void IDrawTool::toolCreatItemUpdate(IDrawTool::CDrawToolEvent *event, IDrawTool::ITERecordInfo *pInfo)
+void IDrawTool::toolCreatItemUpdate(CDrawToolEvent *event, IDrawTool::ITERecordInfo *pInfo)
 {
     Q_UNUSED(pInfo)
 
@@ -459,7 +637,7 @@ void IDrawTool::toolCreatItemUpdate(IDrawTool::CDrawToolEvent *event, IDrawTool:
     event->scene()->update();
 }
 
-void IDrawTool::toolCreatItemFinish(IDrawTool::CDrawToolEvent *event, IDrawTool::ITERecordInfo *pInfo)
+void IDrawTool::toolCreatItemFinish(CDrawToolEvent *event, IDrawTool::ITERecordInfo *pInfo)
 {
     Q_UNUSED(pInfo)
 
@@ -474,17 +652,12 @@ void IDrawTool::clearITE()
     _allITERecordInfo.clear();
 }
 
-void IDrawTool::setViewToSelectionTool(CGraphicsView *pView)
+void IDrawTool::setViewToSelectionTool(PageView *pView)
 {
-    if (pView == nullptr) {
-        pView = CManageViewSigleton::GetInstance()->getCurView();
-    }
+    Q_UNUSED(pView)
+    auto pSelectTool = CDrawToolManagerSigleton::GetInstance()->getDrawTool(selection);
+    pSelectTool->activeTool();
 
-    if (pView != nullptr) {
-        pView->getDrawParam()->setCurrentDrawToolMode(selection);
-        emit pView->drawScene()->signalChangeToSelect();
-        pView->drawScene()->selectGroup()->updateAttributes();
-    }
 }
 
 CGraphicsItem *IDrawTool::creatItem(CDrawToolEvent *event, ITERecordInfo *pInfo)
@@ -499,23 +672,28 @@ int IDrawTool::minMoveUpdateDistance()
     return dApp->startDragDistance();
 }
 
-void IDrawTool::mouseHoverEvent(IDrawTool::CDrawToolEvent *event)
+bool IDrawTool::blockPageBeforeOutput(Page *page)
 {
-    // 只有鼠标才存在hover事件
-    Q_UNUSED(event)
+    Q_UNUSED(page)
+    return false;
 }
 
-void IDrawTool::toolDoubleClikedEvent(IDrawTool::CDrawToolEvent *event, ITERecordInfo *pInfo)
+void IDrawTool::mouseHoverEvent(CDrawToolEvent *event)
+{
+    // 只有鼠标才存在hover事件
+    event->scene()->setCursor(cursor());
+}
+
+void IDrawTool::toolDoubleClikedEvent(CDrawToolEvent *event, ITERecordInfo *pInfo)
 {
     Q_UNUSED(event)
     Q_UNUSED(pInfo)
     qDebug() << "IDrawTool::toolDoubleClikedEvent == " << event->pos() << "tp = " << event->eventType();
 }
 
-void IDrawTool::drawMore(QPainter *painter, const QRectF &rect, CDrawScene *scene)
+void IDrawTool::drawMore(QPainter *painter, const QRectF &rect, PageScene *scene)
 {
     //注意painter是在viewport的坐标系,绘制时需要转换
-
     Q_UNUSED(painter)
     Q_UNUSED(rect)
     Q_UNUSED(scene)
@@ -528,6 +706,9 @@ EDrawToolMode IDrawTool::getDrawToolMode() const
 
 QCursor IDrawTool::getCursor(CSizeHandleRect::EDirection dir, bool bMouseLeftPress, char toolType)
 {
+    Q_UNUSED(dir)
+    Q_UNUSED(bMouseLeftPress)
+    Q_UNUSED(toolType)
 #if 0
     CGraphicsView *pView = CManageViewSigleton::GetInstance()->getCurView();
     if (pView != nullptr) {
@@ -706,6 +887,16 @@ bool IDrawTool::autoSupUndoForCreatItem()
     return true;
 }
 
+bool IDrawTool::rasterItemToLayer(CDrawToolEvent *event, IDrawTool::ITERecordInfo *pInfo)
+{
+    Q_UNUSED(pInfo)
+
+    if (event->keyboardModifiers() & Qt::ControlModifier)
+        return true;
+
+    return false;
+}
+
 void IDrawTool::sendToolEventToItem(CDrawToolEvent *event,
                                     ITERecordInfo *info,
                                     EChangedPhase phase)
@@ -715,46 +906,48 @@ void IDrawTool::sendToolEventToItem(CDrawToolEvent *event,
     Q_UNUSED(phase)
 }
 
+bool IDrawTool::isFirstEvent()
+{
+    return _allITERecordInfo.count() == 1;
+}
+
+bool IDrawTool::isFinalEvent()
+{
+    return _allITERecordInfo.count() == 1;
+}
+
 int IDrawTool::getCurVaildActivedPointCount()
 {
     int result = 0;
-    for (auto it = _allITERecordInfo.begin(); it != _allITERecordInfo.end(); ++it) {
-        if (it.value().eventLife != EDoNotthing) {
+    foreach (auto it, _allITERecordInfo.values()) {
+        if (it.eventLife != EDoNotthing) {
             ++result;
         }
     }
     return result;
 }
 
-bool IDrawTool::isPressEventHandledByQt(IDrawTool::CDrawToolEvent *event, IDrawTool::ITERecordInfo *pInfo)
+void IDrawTool::changeStatusFlagTo(EStatus status)
+{
+    auto oldStatus = this->status();
+    _status = status;
+    if (oldStatus != _status) {
+        emit statusChanged(oldStatus, _status);
+    }
+}
+
+bool IDrawTool::isPressEventHandledByQt(CDrawToolEvent *event, IDrawTool::ITERecordInfo *pInfo)
 {
-    Q_UNUSED(event)
-    Q_UNUSED(pInfo)
     //1.节点node不用传递给qgraphics的qt框架 自己处理调整图元大小
     QGraphicsItem *pItem = pInfo->startPosItems.isEmpty() ? nullptr : pInfo->startPosItems.first();
     if (event->scene()->isBussizeHandleNodeItem(pItem)) {
         return false;
     }
 
-
-
-//    QGraphicsItem *pFocusItem = event->scene()->focusItem();
-//    bool b = (pFocusItem != nullptr &&
-//              pFocusItem->type() == QGraphicsProxyWidget::Type);
-//    return b;
     //2.如果不是节点，那么如果是代理的widget那么传递给qgraphics的qt框架，将事件传递给这个widget
     return (event->view()->activeProxWidget() != nullptr);
 }
 
-//IDrawTool::ITERecordInfo *IDrawTool::getEventIteInfo(int uuid)
-//{
-//    auto itf = _allITERecordInfo.find(uuid);
-//    if (itf != _allITERecordInfo.end()) {
-//        return &itf.value();
-//    }
-//    return nullptr;
-//}
-
 QTimer *IDrawTool::getTimerForDoubleCliked()
 {
     if (s_timerForDoubleClike == nullptr) {
@@ -764,186 +957,7 @@ QTimer *IDrawTool::getTimerForDoubleCliked()
     return s_timerForDoubleClike;
 }
 
-IDrawTool::CDrawToolEvent::CDrawToolEvent(const QPointF &vPos,
-                                          const QPointF &scenePos,
-                                          const QPointF &globelPos,
-                                          CDrawScene *pScene)
-{
-    _pos[EViewportPos] = vPos;
-    _pos[EScenePos]    = scenePos;
-    _pos[EGlobelPos]   = globelPos;
-    _scene             = pScene;
-}
 
-IDrawTool::CDrawToolEvent::CDrawToolEvents IDrawTool::CDrawToolEvent::fromQEvent(QEvent *event, CDrawScene *scene)
-{
-    CDrawToolEvents eList;
-    CDrawToolEvent e;
-    switch (event->type()) {
-    case QEvent::MouseButtonPress:
-    case QEvent::MouseButtonRelease:
-    case QEvent::MouseMove:
-    case QEvent::MouseButtonDblClick: {
-        QMouseEvent *msEvent = dynamic_cast<QMouseEvent *>(event);
-        //e._pos[EViewportPos] = msEvent->pos();
-        e._pos[EViewportPos] = /*msEvent->pos()*/scene->drawView()->viewport()->mapFromGlobal(msEvent->globalPos());
-        e._pos[EGlobelPos]   = msEvent->globalPos();
-        e._msBtns = msEvent->button() | msEvent->buttons();
-        e._kbMods = msEvent->modifiers();
-        e._orgEvent = event;
-        e._scene    = scene;
-        eList.insert(0, e);
-        break;
-    }
-    case QEvent::GraphicsSceneMouseMove:
-    case QEvent::GraphicsSceneMousePress:
-    case QEvent::GraphicsSceneMouseRelease:
-    case QEvent::GraphicsSceneMouseDoubleClick: {
-        QGraphicsSceneMouseEvent *msEvent = dynamic_cast<QGraphicsSceneMouseEvent *>(event);
-        e._pos[EViewportPos] = /*msEvent->pos()*/scene->drawView()->viewport()->mapFromGlobal(msEvent->screenPos());
-        e._pos[EScenePos]    = msEvent->scenePos();
-        e._pos[EGlobelPos]   = msEvent->screenPos();
-        e._msBtns = msEvent->button() | msEvent->buttons();
-        e._kbMods = msEvent->modifiers();
-        e._orgEvent = event;
-        e._scene    = scene;
-        eList.insert(0, e);
-        break;
-    }
-    case QEvent::TouchBegin:
-    case QEvent::TouchUpdate:
-    case QEvent::TouchEnd: {
-        QTouchEvent *touchEvent = dynamic_cast<QTouchEvent *>(event);
-        const QList<QTouchEvent::TouchPoint> lists = touchEvent->touchPoints();
-        for (auto tPos : lists) {
-            e = fromTouchPoint(tPos, scene);
-            e._orgEvent = event;
-            eList.insert(e.uuid(), e);
-        }
-        break;
-    }
-    default:
-        break;
-    }
-    return eList;
-}
-
-IDrawTool::CDrawToolEvent IDrawTool::CDrawToolEvent::fromTouchPoint(const QTouchEvent::TouchPoint &tPos,
-                                                                    CDrawScene *scene, QEvent *eOrg)
-{
-    CDrawToolEvent e;
-    e._pos[EViewportPos] = tPos.pos();
-    e._pos[EScenePos]    = tPos.scenePos();
-    e._pos[EGlobelPos]   = tPos.screenPos();
-    e._uuid              = tPos.id();
-    e._scene             = scene;
-    e._kbMods            = dApp->keyboardModifiers();
-    e._orgEvent = eOrg;
-    //qDebug() << "e._pos[EViewportPos] = " << e._pos[EViewportPos] << "e._pos[EScenePos] = " << e._pos[EScenePos] << "e._pos[EGlobelPos] = " << e._pos[EGlobelPos];
-    return e;
-}
-
-QPointF IDrawTool::CDrawToolEvent::pos(IDrawTool::CDrawToolEvent::EPosType tp) const
-{
-    if (tp >= EScenePos && tp < PosTypeCount) {
-        return _pos[tp];
-    }
-    return QPointF(0, 0);
-}
-
-Qt::MouseButtons IDrawTool::CDrawToolEvent::mouseButtons() const
-{
-    return _msBtns;
-}
-
-Qt::KeyboardModifiers IDrawTool::CDrawToolEvent::keyboardModifiers() const
-{
-    return _kbMods;
-}
-
-int IDrawTool::CDrawToolEvent::uuid() const
-{
-    return _uuid;
-}
-
-IDrawTool::CDrawToolEvent::EEventTp IDrawTool::CDrawToolEvent::eventType()
-{
-    if (orgQtEvent() == nullptr)
-        return EEventSimulated;
-
-    EEventTp tp = EEventSimulated;
-
-    switch (orgQtEvent()->type()) {
-    case QEvent::MouseButtonPress:
-    case QEvent::MouseButtonRelease:
-    case QEvent::MouseMove:
-    case QEvent::MouseButtonDblClick:
-    case QEvent::GraphicsSceneMouseMove:
-    case QEvent::GraphicsSceneMousePress:
-    case QEvent::GraphicsSceneMouseRelease:
-    case QEvent::GraphicsSceneMouseDoubleClick: {
-        tp = EMouseEvent;
-        break;
-    }
-    case QEvent::TouchBegin:
-    case QEvent::TouchUpdate:
-    case QEvent::TouchEnd: {
-        tp = ETouchEvent;
-        break;
-    }
-    default:
-        break;
-    }
-    return tp;
-}
-
-QEvent *IDrawTool::CDrawToolEvent::orgQtEvent()
-{
-    return _orgEvent;
-}
-
-CDrawScene *IDrawTool::CDrawToolEvent::scene()
-{
-    return _scene;
-}
-
-CGraphicsView *IDrawTool::CDrawToolEvent::view() const
-{
-    if (_scene != nullptr) {
-        return _scene->drawView();
-    }
-    return nullptr;
-}
-
-bool IDrawTool::CDrawToolEvent::isAccepted() const
-{
-    return _accept;
-}
-
-void IDrawTool::CDrawToolEvent::setAccepted(bool b)
-{
-    _accept = b;
-}
-
-bool IDrawTool::CDrawToolEvent::isPosXAccepted() const
-{
-    return _acceptPosX;
-}
-
-//void IDrawTool::CDrawToolEvent::setPosXAccepted(bool b)
-//{
-//    _acceptPosX = b;
-//}
-
-bool IDrawTool::CDrawToolEvent::isPosYAccepted() const
-{
-    return _acceptPosY;
-}
-
-//void IDrawTool::CDrawToolEvent::setPosYAccepted(bool b)
-//{
-//    _acceptPosY = b;
-//}
 
 bool IDrawTool::ITERecordInfo::isVaild() const
 {
