@@ -26,26 +26,29 @@ CIntValidator::CIntValidator(int bottom, int top, QObject *parent)
 
 }
 
-
-CIntValidator::CIntValidator(QObject *parent)
-    : QIntValidator(parent)
+void CIntValidator::setEmptyStrToBottom(bool b)
 {
-
+    m_emptyStrIsBottom = b;
 }
 
-CIntValidator::~CIntValidator()
-{
-
-}
+//bool CIntValidator::emptyStrIsBottom() const
+//{
+//    return m_emptyStrIsBottom;
+//}
 
 QValidator::State CIntValidator::validate(QString &s, int &n) const
 {
+    if (s.isEmpty() && !m_emptyStrIsBottom)
+        return QValidator::Acceptable;
+
     return QIntValidator::validate(s, n);
 }
 
-
 void CIntValidator::fixup(QString &str) const
 {
+    if (str.isEmpty() && !m_emptyStrIsBottom) {
+        return;
+    }
     str = QString("%1").arg(bottom());
     QIntValidator::fixup(str);
 }
