@@ -26,12 +26,17 @@
 class CIntValidator : public QIntValidator
 {
 public:
-    explicit CIntValidator(QObject *parent = nullptr);
     explicit CIntValidator(int bottom, int top, QObject *parent = nullptr);
-    ~CIntValidator()override;
+
+    void setEmptyStrToBottom(bool b);
+    //bool emptyStrIsBottom()const;
 
     virtual State validate(QString &, int &) const override;
     virtual void fixup(QString &) const override;
+
+
+private:
+    bool m_emptyStrIsBottom = true;
 };
 
 #endif // CVALIDATOR_H
