@@ -24,7 +24,7 @@
 #include "dialog.h"
 
 
-class DrawDialog : public Dialog
+class DrawDialog : public DDialog
 {
     Q_OBJECT
 public:
@@ -38,7 +38,7 @@ signals:
 
 protected:
     void keyPressEvent(QKeyEvent *e) override;
-
+    void showEvent(QShowEvent *event) override;
 
 };
 
